@@ -209,11 +209,11 @@ unsigned IsJava (unsigned EvType)
 /******************************************************************************
  ***  IsCUDA
  ******************************************************************************/
-#define CUDA_EVENTS 7
+#define CUDA_EVENTS 8
 static unsigned cuda_events[] = {
 	/* Host events */
 	CUDACALL_EV, CUDACALLGPU_EV, CUDA_DYNAMIC_MEM_PTR_EV, CUDA_DYNAMIC_MEM_SIZE_EV,
-	CUDAFUNC_EV, CUDASTREAMBARRIER_THID_EV, CUDA_UNTRACKED_EV
+	CUDA_KERNEL_INST_EV, CUDASTREAMBARRIER_THID_EV, CUDA_KERNEL_EXEC_EV, CUDA_UNTRACKED_EV
 };
 
 unsigned IsCUDA (unsigned EvType)
@@ -229,11 +229,11 @@ unsigned IsCUDA (unsigned EvType)
 /******************************************************************************
  ***  IsHIP
  ******************************************************************************/
-#define HIP_EVENTS 7
+#define HIP_EVENTS 8
 static unsigned hip_events[] = {
 	/* Host events */
 	HIPCALL_EV, HIPCALLGPU_EV, HIP_DYNAMIC_MEM_PTR_EV, HIP_DYNAMIC_MEM_SIZE_EV,
-	HIPFUNC_EV, HIPSTREAMBARRIER_THID_EV, HIP_UNTRACKED_EV
+	HIP_KERNEL_INST_EV, HIPSTREAMBARRIER_THID_EV, HIP_KERNEL_EXEC_EV, HIP_UNTRACKED_EV
 };
 
 unsigned IsHIP (unsigned EvType)
