@@ -601,19 +601,22 @@ enum {
 	CUDAKERNEL_GPU_VAL        = 1,
 	CUDACONFIGKERNEL_GPU_VAL,
 	CUDAMEMCPY_GPU_VAL,
-	CUDATHREADBARRIER_GPU_VAL,
-	CUDASTREAMBARRIER_GPU_VAL,
-	CUDAMEMCPYASYNC_GPU_VAL   = 7
+	CUDAMEMCPYASYNC_GPU_VAL   = 7,
+	CUDAEVENTRECORD_GPU_VAL = 35
 };
 
 #define CUDA_UNTRACKED_EV             63000003
 #define CUDA_DYNAMIC_MEM_SIZE_EV      63000004
 #define CUDA_DYNAMIC_MEM_PTR_EV       63000005
-#define CUDAFUNC_EV                   63000006
-#define CUDAFUNC_LINE_EV              63000007
-
-/* To associate stream with Paraver thread */
-#define CUDASTREAMBARRIER_THID_EV     63000008
+#define CUDA_KERNEL_INST_EV           63000006
+#define CUDA_KERNEL_INST_LINE_EV      63000007
+#define CUDASTREAMBARRIER_THID_EV     63000008  /* TID of the Paraver thid of the CUDA stream being synchronized */
+#define CUDA_KERNEL_EXEC_EV           63000009
+#define CUDA_KERNEL_EXEC_LINE_EV      63000010
+#define CUDA_KERNEL_BLOCKS_PER_GRID   63000011
+#define CUDA_KERNEL_THREADS_PER_BLOCK 63000012
+#define CUDA_KERNEL_SHARED_MEM_BYTES  63000013
+#define CUDA_MEMORY_TRANSFER          63000014
 
 #define HIPCALL_EV               63500000
 enum {
@@ -652,8 +655,7 @@ enum {
 	HIPMEMCPYFROMSYMBOLASYNC_VAL,  /*TBD*/
 	HIPMEMSET_VAL,
 	HIPEVENTRECORD_VAL,
-	HIPEVENTSYNCHRONIZE_VAL,
-	HIPGETSYMBOLADDRESS_VAL
+	HIPEVENTSYNCHRONIZE_VAL
 };
 
 #define HIPCALLGPU_EV       63500001
@@ -661,19 +663,22 @@ enum {
 	HIPKERNEL_GPU_VAL        = 1,
 	HIPCONFIGKERNEL_GPU_VAL,
 	HIPMEMCPY_GPU_VAL,
-	HIPTHREADBARRIER_GPU_VAL,
-	HIPSTREAMBARRIER_GPU_VAL,
-	HIPMEMCPYASYNC_GPU_VAL   = 7
+	HIPMEMCPYASYNC_GPU_VAL   = 7,
+	HIPEVENTRECORD_GPU_VAL = 35
 };
 
 #define HIP_UNTRACKED_EV             63500003
 #define HIP_DYNAMIC_MEM_SIZE_EV      63500004
 #define HIP_DYNAMIC_MEM_PTR_EV       63500005
-#define HIPFUNC_EV                   63500006
-#define HIPFUNC_LINE_EV              63500007
-
-/* To associate stream with Paraver thread */
-#define HIPSTREAMBARRIER_THID_EV     63500008
+#define HIP_KERNEL_INST_EV           63500006
+#define HIP_KERNEL_INST_LINE_EV      63500007
+#define HIPSTREAMBARRIER_THID_EV     63500008  /* TID of the Paraver thid of the HIP stream being synchronized */
+#define HIP_KERNEL_EXEC_EV           63500009
+#define HIP_KERNEL_EXEC_LINE_EV      63500010
+#define HIP_KERNEL_BLOCKS_PER_GRID   63500011
+#define HIP_KERNEL_THREADS_PER_BLOCK 63500012
+#define HIP_KERNEL_SHARED_MEM_BYTES  63500013
+#define HIP_MEMORY_TRANSFER          63500014
 
 #define OPENCL_KERNEL_NAME_EV                         64200000
 #define OPENCL_BASE_TYPE_EV                           64000000
