@@ -427,25 +427,9 @@ static void AddressTable_Insert (UINT64 address, int addr_type,
  * @param filename The source file name
  * @param line The line number in the source file
  */
-void Address2Info_AddSymbol (UINT64 address, int addr_type, char * funcname, char * filename, int line)
+void Address2Info_AddSymbol (UINT64 address, int addr_type,
+                             char * funcname, char * filename, int line)
 {
-
-	//if (!Address2Info_Initialized()) 
-	//{
-    //    Address2Info_Initialize();
-    //}
-
-	//fprintf(stderr, "address: %llu\n", (unsigned long long)address);
-	//fflush(stderr);
-	//fprintf(stderr, "addr_type: %d\n", addr_type);
-	//fflush(stderr);
-	//fprintf(stderr,"funcname: %s\n", funcname);
-	//fflush(stderr);
-	//fprintf(stderr,"filename: %s\n", filename);
-	//fflush(stderr);
-	//fprintf(stderr,"line: %d\n", line);
-	//fflush(stderr);
-
 	int found = FALSE;
 	int i = 0;
 
@@ -1496,7 +1480,8 @@ void Address2Info_Write_CUDA_Labels (FILE * pcf_fd, int uniqueid)
 	if (Address2Info_Labels[A2I_CUDA]) 
 	{
 		fprintf (pcf_fd, "%s\n", TYPE_LABEL);
-		fprintf (pcf_fd, "0    %d    %s\n", CUDAFUNC_EV, "CUDA kernel");
+		fprintf (pcf_fd, "0    %d    %s\n", CUDA_KERNEL_INST_EV, "CUDA kernel instantiation");
+		fprintf (pcf_fd, "0    %d    %s\n", CUDA_KERNEL_EXEC_EV, "CUDA kernel execution");
 
 		if (Address2Info_Initialized())
 		{
@@ -1520,7 +1505,9 @@ void Address2Info_Write_CUDA_Labels (FILE * pcf_fd, int uniqueid)
 
 		/* Then dump line-functions */
 		fprintf (pcf_fd, "%s\n", TYPE_LABEL);
-		fprintf (pcf_fd, "0    %d    %s\n", CUDAFUNC_LINE_EV, "CUDA kernel source code line");
+		fprintf (pcf_fd, "0    %d    %s\n", CUDA_KERNEL_INST_LINE_EV, "CUDA kernel instantiation source code line");
+		fprintf (pcf_fd, "0    %d    %s\n", CUDA_KERNEL_EXEC_LINE_EV, "CUDA kernel execution source code line");
+
 		if (Address2Info_Initialized())
 		{
 			fprintf (pcf_fd, "%s\n0 %s\n", VALUES_LABEL, EVT_END_LBL);
@@ -1546,7 +1533,6 @@ void Address2Info_Write_CUDA_Labels (FILE * pcf_fd, int uniqueid)
 
 void Address2Info_Write_HIP_Labels (FILE * pcf_fd, int uniqueid)
 {
-
 	struct address_table  * AddrTab;
 	struct function_table * FuncTab;
 	int i;
@@ -1556,8 +1542,9 @@ void Address2Info_Write_HIP_Labels (FILE * pcf_fd, int uniqueid)
 
 	if (Address2Info_Labels[A2I_HIP]) 
 	{
-		fprintf(pcf_fd, "%s\n", TYPE_LABEL);
-		fprintf (pcf_fd, "0    %d    %s\n", HIPFUNC_EV, "HIP kernel");
+		fprintf (pcf_fd, "%s\n", TYPE_LABEL);
+		fprintf (pcf_fd, "0    %d    %s\n", HIP_KERNEL_INST_EV, "HIP kernel instantiation");
+		fprintf (pcf_fd, "0    %d    %s\n", HIP_KERNEL_EXEC_EV, "HIP kernel execution");
 
 		if (Address2Info_Initialized())
 		{
@@ -1581,7 +1568,9 @@ void Address2Info_Write_HIP_Labels (FILE * pcf_fd, int uniqueid)
 
 		/* Then dump line-functions */
 		fprintf (pcf_fd, "%s\n", TYPE_LABEL);
-		fprintf (pcf_fd, "0    %d    %s\n", HIPFUNC_LINE_EV, "HIP kernel source code line");
+		fprintf (pcf_fd, "0    %d    %s\n", HIP_KERNEL_INST_LINE_EV, "HIP kernel instantiation source code line");
+		fprintf (pcf_fd, "0    %d    %s\n", HIP_KERNEL_EXEC_LINE_EV, "HIP kernel execution source code line");
+
 		if (Address2Info_Initialized())
 		{
 			fprintf (pcf_fd, "%s\n0 %s\n", VALUES_LABEL, EVT_END_LBL);
@@ -1979,4 +1968,4 @@ void Address2Info_Unify(int numtasks, int taskid, struct address_collector_t *Co
 		unify_library_ids();
 	}
 #endif
-}
\ No newline at end of file
+}
