@@ -37,6 +37,7 @@
 #include "threadinfo.h"
 #include "wrapper.h"
 #include "trace_macros.h"
+#include "trace_macros_gpu.h"
 #include "cuda_probe.h"
 #include "xalloc.h"
 #include "extrae_user_events.h"
@@ -53,8 +54,6 @@
  */
 int cudaInitialized = 0;
 
-gpu_event_list_t availableEvents;
-
 /* Structures that will hold the parameters needed for the exit parts of the
  * instrumentation code. This way we can support dyninst/ld-preload/cupti
  * instrumentation with a single file
@@ -93,22 +92,12 @@ typedef union
 
 typedef struct
 {
-	int instrumentationDepth;
 	Extrae_cuda_saved_params_union punion;
 } Extrae_cuda_saved_params_t;
 
 unsigned cuda_events_block_size = DEFAULT_CUDA_EVENTS_BLOCK_SIZE;
 
 static Extrae_cuda_saved_params_t *Extrae_CUDA_saved_params = NULL;
-void Extrae_CUDA_updateDepth_(int step)
-{
-	Extrae_CUDA_saved_params[THREADID].instrumentationDepth += step;
-}
-
-int Extrae_CUDA_getDepth()
-{
-	return Extrae_CUDA_saved_params[THREADID].instrumentationDepth;
-}
 
 static unsigned __last_tag = 0xC0DA; /* Fixed tag */
 static unsigned Extrae_CUDA_tag_generator (void)
@@ -141,7 +130,7 @@ static void Extrae_CUDA_SynchronizeStream (int devid, int streamid)
 
 	if(devices[devid].Stream[streamid].device_reference_event == NULL)
 	{
-		devices[devid].Stream[streamid].device_reference_event = gpuEventList_pop(&availableEvents);
+		devices[devid].Stream[streamid].device_reference_event = gpuEventList_pop(&devices[devid].availableEvents);
 	}
 
 	err = cudaEventRecord (devices[devid].Stream[streamid].device_reference_event->ts_event, devices[devid].Stream[streamid].stream);
@@ -226,7 +215,7 @@ void Extrae_CUDA_Initialize (int devid)
 		/* default device stream */
 		devices[devid].Stream[0].stream = (cudaStream_t) 0;
 		gpuEventList_init(&devices[devid].Stream[0].gpu_event_list, FALSE, XTR_CUDA_EVENTS_BLOCK_SIZE);
-		gpuEventList_init(&availableEvents, TRUE, XTR_CUDA_EVENTS_BLOCK_SIZE);
+		gpuEventList_init(&devices[devid].availableEvents, TRUE, XTR_CUDA_EVENTS_BLOCK_SIZE);
 
 		/* Create an event record and process it through the stream! */
 		/* FIX CU_EVENT_BLOCKING_SYNC may be harmful!? */
@@ -353,17 +342,19 @@ static void Extrae_CUDA_RegisterStream (int devid, cudaStream_t stream)
 
 static void Extrae_CUDA_AddEventToStream (Extrae_CUDA_Time_Type timetype,
 	int devid, int streamid, unsigned event, unsigned long long value,
-	unsigned tag, unsigned size)
+	unsigned tag, size_t size, unsigned int blockspergrid, unsigned int threadsperblock)
 {
 	int err;
 	struct RegisteredStreams_t *registered_stream = &devices[devid].Stream[streamid];
 
-	gpu_event_t* gpu_event = gpuEventList_pop(&availableEvents);
+	gpu_event_t* gpu_event = gpuEventList_pop(&devices[devid].availableEvents);
 
 	gpu_event->event = event;
 	gpu_event->value = value;
 	gpu_event->tag = tag;
-	gpu_event->size = size;
+	gpu_event->memSize = size;
+	gpu_event->blocksPerGrid = blockspergrid;
+	gpu_event->threadsPerBlock = threadsperblock;
 	gpu_event->timetype = timetype;
 
 	err = cudaEventRecord (gpu_event->ts_event, registered_stream->stream);
@@ -372,37 +363,39 @@ static void Extrae_CUDA_AddEventToStream (Extrae_CUDA_Time_Type timetype,
 	gpuEventList_add(&registered_stream->gpu_event_list, gpu_event);
 }
 
-static void traceGPUEvents(int threadid, UINT64 time, unsigned event, unsigned long long value, unsigned size, unsigned tag)
+/**
+ * This routine emits GPU events into the tracing buffer and
+ * inserts communication events to connect host-side 
+ * CUDA runtime calls with their corresponding GPU kernel and memcopies executions.
+ *
+ * @param threadid          buffer thread id
+ * @param time              Timestamp of the event
+ * @param event             kind of event (CUDAKERNEL_GPU_VAL, CUDAMEMCPY_GPU_VAL...)
+ * @param value             Entry/exit (EVT_BEGIN/EVT_END)
+ * @param tag               Communication tag
+ * @param size              Size in bytes for memory operations
+ * @param blockspergrid     Grid configuration (CUDA kernels)
+ * @param threadsperblock   Block configuration (CUDA kernels)
+ */
+static void traceGPUEvents(int threadid, UINT64 time, unsigned event, unsigned long long value, unsigned tag, size_t size, unsigned blockspergrid, unsigned threadsperblock)
 {
-	/* Emit events into the tracing buffer. */
-	THREAD_TRACE_MISCEVENT (threadid, time, CUDACALLGPU_EV, event, value);
+	TRACE_GPU_EVENT(threadid, time, CUDACALLGPU_EV, event, value, size);
 
 	if (event == CUDAKERNEL_GPU_VAL)
 	{
-		THREAD_TRACE_MISCEVENT(threadid, time, CUDAFUNC_EV, value, 0);
-	}
-
-	/* Emit communication records for memory transfer, kernel setup and kernel execution */
-	if (event == CUDAMEMCPY_GPU_VAL || event == CUDAMEMCPYASYNC_GPU_VAL)
-	{
-		if (value != EVT_END)
+		TRACE_GPU_KERNEL_EVENT(threadid, time, CUDA_KERNEL_EXEC_EV, value, blockspergrid, threadsperblock);
+		if(value != EVT_END) 
 		{
-			THREAD_TRACE_MISCEVENT(threadid, time, CUDA_DYNAMIC_MEM_SIZE_EV, size, 0);
+			THREAD_TRACE_USER_COMMUNICATION_EVENT(threadid, time,USER_RECV_EV,TASKID,0, tag, tag);
 		}
-
-		if (tag > 0)
+	}
+	else
+	{
+		if ((event == CUDAMEMCPY_GPU_VAL || event == CUDAMEMCPYASYNC_GPU_VAL) && (tag > 0))
 		{
 			THREAD_TRACE_USER_COMMUNICATION_EVENT(threadid, time, (value==EVT_END)?USER_RECV_EV:USER_SEND_EV, TASKID, size, tag, tag);
 		}
 	}
-	else if (event == CUDAKERNEL_GPU_VAL && value != EVT_END)
-	{
-		THREAD_TRACE_USER_COMMUNICATION_EVENT(threadid, time,USER_RECV_EV,TASKID,0, tag, tag);
-	}
-	else if (event == CUDACONFIGKERNEL_GPU_VAL && value != EVT_END)
-	{
-		THREAD_TRACE_USER_COMMUNICATION_EVENT(threadid, time, USER_RECV_EV, TASKID, 0, tag, tag);
-	}
 }
 
 static void flushGPUEvents (int devid, int streamid)
@@ -451,20 +444,20 @@ static void flushGPUEvents (int devid, int streamid)
 				utmp = last_time;
 			}
 
-			traceGPUEvents(threadid, utmp, gpu_event->event, gpu_event->value, gpu_event->size, gpu_event->tag);
+			traceGPUEvents(threadid, utmp, gpu_event->event, gpu_event->value, gpu_event->tag, gpu_event->memSize, gpu_event->blocksPerGrid, gpu_event->threadsPerBlock);
 
 			last_time = utmp;
 
 			if(gpuEventList_isempty(&registered_stream->gpu_event_list))
 			{
-				gpuEventList_add(&availableEvents, registered_stream->device_reference_event);
+				gpuEventList_add(&devices[devid].availableEvents, registered_stream->device_reference_event);
 				registered_stream->device_reference_event = gpu_event;
 				registered_stream->host_reference_time = last_time;
 				gpu_event = NULL;
 			}
 			else
 			{
-				gpuEventList_add(&availableEvents, gpu_event);
+				gpuEventList_add(&devices[devid].availableEvents, gpu_event);
 				gpu_event = gpuEventList_pop(&registered_stream->gpu_event_list);
 			}
 		}
@@ -490,7 +483,6 @@ void Extrae_cudaConfigureCall_Enter (dim3 p1, dim3 p2, size_t p3, cudaStream_t p
 	cudaGetDevice (&devid);
 	Extrae_CUDA_Initialize (devid);
 
-	Backend_Enter_Instrumentation ();
 	Probe_Cuda_ConfigureCall_Entry ();
 
 	TRACE_USER_COMMUNICATION_EVENT (LAST_READ_TIME, USER_SEND_EV, TASKID, 0, tag, tag);
@@ -503,7 +495,7 @@ void Extrae_cudaConfigureCall_Enter (dim3 p1, dim3 p2, size_t p3, cudaStream_t p
 	}
 	_cudaLaunch_stream = strid;
 	Extrae_CUDA_AddEventToStream(EXTRAE_CUDA_NEW_TIME, devid, strid,
-	  CUDACONFIGKERNEL_GPU_VAL, EVT_BEGIN, tag, tag);
+	  CUDACONFIGKERNEL_GPU_VAL, EVT_BEGIN, tag, tag, 0, 0);
 }
 
 void Extrae_cudaConfigureCall_Exit (void)
@@ -513,13 +505,16 @@ void Extrae_cudaConfigureCall_Exit (void)
 	cudaGetDevice (&devid);
 
 	Extrae_CUDA_AddEventToStream(EXTRAE_CUDA_NEW_TIME, devid, _cudaLaunch_stream, 
-	  CUDACONFIGKERNEL_GPU_VAL, EVT_END, Extrae_CUDA_tag_get(), 0);
+	  CUDACONFIGKERNEL_GPU_VAL, EVT_END, Extrae_CUDA_tag_get(), 0, 0, 0);
 
 	Probe_Cuda_ConfigureCall_Exit ();
-	Backend_Leave_Instrumentation ();
 }
 
-void Extrae_cudaLaunch_Enter (const char *p1, cudaStream_t stream)
+void Extrae_cudaLaunch_Enter (const char *f,
+    unsigned int blocksPerGrid,
+    unsigned int threadsPerBlock,
+    size_t sharedMemBytes,
+    cudaStream_t stream)
 {
 	int devid;
 	unsigned tag = Extrae_CUDA_tag_generator();
@@ -527,8 +522,7 @@ void Extrae_cudaLaunch_Enter (const char *p1, cudaStream_t stream)
 	cudaGetDevice (&devid);
 	Extrae_CUDA_Initialize (devid);
 
-	Backend_Enter_Instrumentation ();
-	Probe_Cuda_Launch_Entry ((UINT64) p1);
+	Probe_Cuda_Launch_Entry ((UINT64) f);
 
 	TRACE_USER_COMMUNICATION_EVENT (LAST_READ_TIME, USER_SEND_EV, TASKID, 0, tag, tag);
 
@@ -536,7 +530,7 @@ void Extrae_cudaLaunch_Enter (const char *p1, cudaStream_t stream)
 	_cudaLaunch_stream = strid;
 
 	Extrae_CUDA_AddEventToStream(EXTRAE_CUDA_NEW_TIME, devid, _cudaLaunch_stream,
-	  CUDAKERNEL_GPU_VAL, (UINT64)p1, tag, tag);
+	  CUDAKERNEL_GPU_VAL, (UINT64)f, tag, sharedMemBytes, blocksPerGrid, threadsPerBlock);
 }
 
 void Extrae_cudaLaunch_Exit (void)
@@ -547,10 +541,9 @@ void Extrae_cudaLaunch_Exit (void)
 	Extrae_CUDA_Initialize (devid);
 
 	Extrae_CUDA_AddEventToStream(EXTRAE_CUDA_NEW_TIME, devid, _cudaLaunch_stream,
-	  CUDAKERNEL_GPU_VAL, EVT_END, Extrae_CUDA_tag_get(), 0);
+	  CUDAKERNEL_GPU_VAL, EVT_END, Extrae_CUDA_tag_get(), 0, 0, 0);
 
 	Probe_Cuda_Launch_Exit ();
-	Backend_Leave_Instrumentation ();
 }
 
 void Extrae_cudaMalloc_Enter(unsigned int event, void **devPtr, size_t size)
@@ -560,7 +553,6 @@ void Extrae_cudaMalloc_Enter(unsigned int event, void **devPtr, size_t size)
 	cudaGetDevice(&devid);
 	Extrae_CUDA_Initialize(devid);
 
-	Backend_Enter_Instrumentation();
 	Probe_Cuda_Malloc_Entry(event, (UINT64)devPtr, size);
 }
 
@@ -572,7 +564,6 @@ void Extrae_cudaMalloc_Exit(unsigned int event)
 	Extrae_CUDA_Initialize(devid);
 
 	Probe_Cuda_Malloc_Exit(event);
-	Backend_Leave_Instrumentation();
 }
 
 void Extrae_cudaFree_Enter(unsigned int event, void *devPtr)
@@ -582,7 +573,6 @@ void Extrae_cudaFree_Enter(unsigned int event, void *devPtr)
 	cudaGetDevice(&devid);
 	Extrae_CUDA_Initialize(devid);
 
-	Backend_Enter_Instrumentation();
 	Probe_Cuda_Free_Entry(event, (UINT64)devPtr);
 }
 
@@ -594,19 +584,16 @@ void Extrae_cudaFree_Exit(unsigned int event)
 	Extrae_CUDA_Initialize(devid);
 
 	Probe_Cuda_Free_Exit(event);
-	Backend_Leave_Instrumentation();
 }
 
 void Extrae_cudaHostAlloc_Enter(void **pHost, size_t size)
 {
-	Backend_Enter_Instrumentation();
 	Probe_Cuda_HostAlloc_Entry((UINT64)pHost, size);
 }
 
 void Extrae_cudaHostAlloc_Exit()
 {
 	Probe_Cuda_HostAlloc_Exit();
-	Backend_Leave_Instrumentation();
 }
 
 void Extrae_cudaDeviceSynchronize_Enter (void)
@@ -616,7 +603,6 @@ void Extrae_cudaDeviceSynchronize_Enter (void)
 	cudaGetDevice (&devid);
 	Extrae_CUDA_Initialize (devid);
 
-	Backend_Enter_Instrumentation ();
 	Probe_Cuda_ThreadBarrier_Entry ();
 	Extrae_CUDA_flush_streams(devid, XTR_FLUSH_ALL_STREAMS);
 }
@@ -628,7 +614,6 @@ void Extrae_cudaDeviceSynchronize_Exit (void)
 	cudaGetDevice (&devid);
 
 	Probe_Cuda_ThreadBarrier_Exit ();
-	Backend_Leave_Instrumentation ();
 }
 
 void Extrae_cudaThreadSynchronize_Enter (void)
@@ -638,8 +623,8 @@ void Extrae_cudaThreadSynchronize_Enter (void)
 	cudaGetDevice (&devid);
 	Extrae_CUDA_Initialize (devid);
 
-	Backend_Enter_Instrumentation ();
 	Probe_Cuda_ThreadBarrier_Entry ();
+	Extrae_CUDA_flush_streams(devid, XTR_FLUSH_ALL_STREAMS);
 }
 
 void Extrae_cudaThreadSynchronize_Exit (void)
@@ -649,8 +634,6 @@ void Extrae_cudaThreadSynchronize_Exit (void)
 	cudaGetDevice (&devid);
 
 	Probe_Cuda_ThreadBarrier_Exit ();
-	Extrae_CUDA_flush_streams(devid, XTR_FLUSH_ALL_STREAMS);
-	Backend_Leave_Instrumentation ();
 }
 
 void Extrae_CUDA_flush_streams ( int device_id, int stream_id )
@@ -684,7 +667,6 @@ void Extrae_cudaStreamCreate_Enter (cudaStream_t *p1)
 
 	Extrae_CUDA_saved_params[THREADID].punion.csc.stream = p1;
 
-	Backend_Enter_Instrumentation ();
 	Probe_Cuda_StreamCreate_Entry ();
 }
 
@@ -701,7 +683,6 @@ void Extrae_cudaStreamCreate_Exit (void)
 	  *Extrae_CUDA_saved_params[THREADID].punion.csc.stream);
 
 	Probe_Cuda_StreamCreate_Exit ();
-	Backend_Leave_Instrumentation ();
 }
 
 void Extrae_cudaStreamDestroy_Enter (cudaStream_t stream)
@@ -710,7 +691,6 @@ void Extrae_cudaStreamDestroy_Enter (cudaStream_t stream)
 
 	ASSERT(Extrae_CUDA_saved_params!=NULL, "Unallocated Extrae_CUDA_saved_params");
 
-	Backend_Enter_Instrumentation ();
 	Probe_Cuda_StreamDestroy_Entry ();
 
 	Extrae_CUDA_saved_params[THREADID].punion.css.stream = stream;
@@ -726,7 +706,6 @@ void Extrae_cudaStreamDestroy_Exit (void)
 	ASSERT(Extrae_CUDA_saved_params!=NULL, "Unallocated Extrae_CUDA_saved_params");
 
 	Probe_Cuda_StreamDestroy_Exit ();
-	Backend_Leave_Instrumentation ();
 }
 
 void Extrae_cudaStreamSynchronize_Enter (cudaStream_t p1)
@@ -744,7 +723,6 @@ void Extrae_cudaStreamSynchronize_Enter (cudaStream_t p1)
 	strid = Extrae_CUDA_SearchStream (devid,
 	  Extrae_CUDA_saved_params[THREADID].punion.css.stream);
 
-	Backend_Enter_Instrumentation ();
 	Probe_Cuda_StreamBarrier_Entry (devices[devid].Stream[strid].threadid);
 
 	Extrae_CUDA_flush_streams(devid, strid);
@@ -775,8 +753,6 @@ void Extrae_cudaStreamSynchronize_Exit (void)
 	}
 
 	Probe_Cuda_StreamBarrier_Exit ();
-
-	Backend_Leave_Instrumentation ();
 }
 
 void Extrae_cudaMemcpy_Enter (void* p1, const void* p2, size_t p3, enum cudaMemcpyKind p4)
@@ -795,7 +771,6 @@ void Extrae_cudaMemcpy_Enter (void* p1, const void* p2, size_t p3, enum cudaMemc
 	cudaGetDevice (&devid);
 	Extrae_CUDA_Initialize (devid);
 
-	Backend_Enter_Instrumentation ();
 	Probe_Cuda_Memcpy_Entry (p3);
 
 	tag = Extrae_CUDA_tag_generator();
@@ -816,12 +791,12 @@ void Extrae_cudaMemcpy_Enter (void* p1, const void* p2, size_t p3, enum cudaMemc
 	if (p4 == cudaMemcpyHostToDevice)
 	{
 		Extrae_CUDA_AddEventToStream(EXTRAE_CUDA_NEW_TIME, devid, 0,
-		  CUDAMEMCPY_GPU_VAL, EVT_BEGIN, 0, p3);
+		  CUDAMEMCPY_GPU_VAL, EVT_BEGIN, 0, p3, 0, 0);
 	}
 	else if (p4 == cudaMemcpyDeviceToHost)
 	{
 		Extrae_CUDA_AddEventToStream(EXTRAE_CUDA_NEW_TIME, devid, 0,
-		  CUDAMEMCPY_GPU_VAL, EVT_BEGIN, tag, p3);
+		  CUDAMEMCPY_GPU_VAL, EVT_BEGIN, tag, p3, 0, 0);
 	}
 }
 
@@ -848,12 +823,12 @@ void Extrae_cudaMemcpy_Exit (void)
 	  Extrae_CUDA_saved_params[THREADID].punion.cm.kind == cudaMemcpyDeviceToDevice)
 	{
 		Extrae_CUDA_AddEventToStream(EXTRAE_CUDA_NEW_TIME, devid, 0,
-		  CUDAMEMCPY_GPU_VAL, EVT_END, tag, Extrae_CUDA_saved_params[THREADID].punion.cm.size);
+		  CUDAMEMCPY_GPU_VAL, EVT_END, tag, Extrae_CUDA_saved_params[THREADID].punion.cm.size, 0, 0);
 	}
 	else if (Extrae_CUDA_saved_params[THREADID].punion.cm.kind == cudaMemcpyDeviceToHost)
 	{
 		Extrae_CUDA_AddEventToStream(EXTRAE_CUDA_NEW_TIME, devid, 0,
-		  CUDAMEMCPY_GPU_VAL, EVT_END, 0, Extrae_CUDA_saved_params[THREADID].punion.cm.size);
+		  CUDAMEMCPY_GPU_VAL, EVT_END, 0, Extrae_CUDA_saved_params[THREADID].punion.cm.size, 0, 0);
 	}
 
 	Probe_Cuda_Memcpy_Exit ();
@@ -864,8 +839,6 @@ void Extrae_cudaMemcpy_Exit (void)
 		TRACE_USER_COMMUNICATION_EVENT(LAST_READ_TIME, USER_RECV_EV,
 		  TASKID, Extrae_CUDA_saved_params[THREADID].punion.cm.size, tag, tag);
 	}
-
-	Backend_Leave_Instrumentation ();
 }
 
 void Extrae_cudaMemcpyAsync_Enter (void* p1, const void* p2, size_t p3, enum cudaMemcpyKind p4,cudaStream_t p5)
@@ -886,7 +859,6 @@ void Extrae_cudaMemcpyAsync_Enter (void* p1, const void* p2, size_t p3, enum cud
 	cudaGetDevice (&devid);
 	Extrae_CUDA_Initialize (devid);
 
-	Backend_Enter_Instrumentation ();
 	Probe_Cuda_MemcpyAsync_Entry (p3);
 
 	tag = Extrae_CUDA_tag_generator();
@@ -914,12 +886,12 @@ void Extrae_cudaMemcpyAsync_Enter (void* p1, const void* p2, size_t p3, enum cud
 	if (p4 == cudaMemcpyHostToDevice)
 	{
 		Extrae_CUDA_AddEventToStream(EXTRAE_CUDA_NEW_TIME, devid, strid,
-		  CUDAMEMCPYASYNC_GPU_VAL, EVT_BEGIN, 0, p3);
+		  CUDAMEMCPYASYNC_GPU_VAL, EVT_BEGIN, 0, p3, 0, 0);
 	}
 	else if (p4 == cudaMemcpyDeviceToHost)
 	{
 		Extrae_CUDA_AddEventToStream(EXTRAE_CUDA_NEW_TIME, devid, strid,
-		  CUDAMEMCPYASYNC_GPU_VAL, EVT_BEGIN, tag, p3);
+		  CUDAMEMCPYASYNC_GPU_VAL, EVT_BEGIN, tag, p3, 0, 0);
 	}
 }
 
@@ -955,12 +927,12 @@ void Extrae_cudaMemcpyAsync_Exit (void)
 	   Extrae_CUDA_saved_params[THREADID].punion.cma.kind == cudaMemcpyDeviceToDevice)
 	{
 		Extrae_CUDA_AddEventToStream(EXTRAE_CUDA_NEW_TIME, devid, strid,
-		  CUDAMEMCPYASYNC_GPU_VAL, EVT_END, tag, Extrae_CUDA_saved_params[THREADID].punion.cma.size);
+		  CUDAMEMCPYASYNC_GPU_VAL, EVT_END, tag, Extrae_CUDA_saved_params[THREADID].punion.cma.size, 0, 0);
 	}
 	else
 	{
 		Extrae_CUDA_AddEventToStream(EXTRAE_CUDA_NEW_TIME, devid, strid,
-		  CUDAMEMCPYASYNC_GPU_VAL, EVT_END, 0, Extrae_CUDA_saved_params[THREADID].punion.cma.size);
+		  CUDAMEMCPYASYNC_GPU_VAL, EVT_END, 0, Extrae_CUDA_saved_params[THREADID].punion.cma.size, 0, 0);
 	}
 
 	Probe_Cuda_MemcpyAsync_Exit ();
@@ -971,8 +943,6 @@ void Extrae_cudaMemcpyAsync_Exit (void)
 		TRACE_USER_COMMUNICATION_EVENT(LAST_READ_TIME, USER_RECV_EV,
 		  TASKID, Extrae_CUDA_saved_params[THREADID].punion.cma.size, tag, tag);
 	}
-
-	Backend_Leave_Instrumentation ();
 }
 
 void Extrae_cudaMemset_Enter(void *devPtr, size_t count)
@@ -982,7 +952,6 @@ void Extrae_cudaMemset_Enter(void *devPtr, size_t count)
 	cudaGetDevice(&devid);
 	Extrae_CUDA_Initialize(devid);
 
-	Backend_Enter_Instrumentation();
 	Probe_Cuda_Memset_Entry((UINT64)devPtr, count);
 }
 
@@ -994,7 +963,6 @@ void Extrae_cudaMemset_Exit()
 	Extrae_CUDA_Initialize(devid);
 
 	Probe_Cuda_Memset_Exit();
-	Backend_Leave_Instrumentation();
 }
 
 void Extrae_reallocate_CUDA_info (unsigned old_threads, unsigned nthreads)
@@ -1013,7 +981,6 @@ void Extrae_reallocate_CUDA_info (unsigned old_threads, unsigned nthreads)
 
 void Extrae_cudaDeviceReset_Enter (void)
 {
-	Backend_Enter_Instrumentation ();
 	Probe_Cuda_DeviceReset_Enter();
 	Extrae_CUDA_flush_streams(XTR_FLUSH_ALL_DEVICES, XTR_FLUSH_ALL_STREAMS);
 }
@@ -1025,12 +992,10 @@ void Extrae_cudaDeviceReset_Exit (void)
 
 	Extrae_CUDA_deInitialize (devid);
 	Probe_Cuda_DeviceReset_Exit();
-	Backend_Leave_Instrumentation ();
 }
 
 void Extrae_cudaThreadExit_Enter (void)
 {
-	Backend_Enter_Instrumentation ();
 	Probe_Cuda_ThreadExit_Enter();
 }
 
@@ -1041,48 +1006,53 @@ void Extrae_cudaThreadExit_Exit (void)
 
 	Extrae_CUDA_deInitialize (devid);
 	Probe_Cuda_ThreadExit_Exit();
-	Backend_Leave_Instrumentation ();
 }
 
 void Extrae_cudaEventRecord_Enter()
 {
-	Backend_Enter_Instrumentation();
 	Probe_Cuda_EventRecord_Entry();
 }
 
 void Extrae_cudaEventRecord_Exit()
 {
 	Probe_Cuda_EventRecord_Exit();
-	Backend_Leave_Instrumentation();
 }
 
 void Extrae_cudaEventSynchronize_Enter()
 {
-	Backend_Enter_Instrumentation();
 	Probe_Cuda_EventSynchronize_Entry();
 }
 
 void Extrae_cudaEventSynchronize_Exit()
 {
 	Probe_Cuda_EventSynchronize_Exit();
-	Backend_Leave_Instrumentation();
 }
 
+/**
+ * Performing the flush of streams events during the finalization of Extrae in the destructor call
+ * triggers CUDA calls after the CUDA library has been unloaded resulting in crashes.
+ * To solve that case we always call Extrae_CUDA_finalize with atexit and mark cudaInitialized to 0
+ * to prevent entering this routine after the return point of the main.
+ */
 void Extrae_CUDA_finalize (void)
 {
+	Backend_Enter_Instrumentation();
 	if (EXTRAE_INITIALIZED() && cudaInitialized == 1 )
 	{
 		Extrae_CUDA_flush_streams (XTR_FLUSH_ALL_DEVICES, XTR_FLUSH_ALL_STREAMS);
 
 		for(int i = 0; i <CUDAdevices ; ++i){
-			Extrae_CUDA_deInitialize (i);
+			if(devices[i].initialized == TRUE){
+				gpuEventList_free(&devices[i].availableEvents);
+				Extrae_CUDA_deInitialize (i);
+			}
 		}
 		xfree(devices);
 
 		devices = NULL;
 		CUDAdevices = 0;
 
-		gpuEventList_free(&availableEvents);
 		cudaInitialized = 0;
 	}
+	Backend_Leave_Instrumentation();
 }
