@@ -145,9 +145,16 @@ Extrae_DriverAPI_callback(CUpti_CallbackId cbid, const CUpti_CallbackData *cbinf
 			  (cuLaunchKernel_params*)cbinfo->functionParams;
 
 			if (cbinfo->callbackSite == CUPTI_API_ENTER)
-				Extrae_cudaLaunch_Enter(p->f, p->hStream);
+			{
+				Extrae_cudaLaunch_Enter(p->f, 
+				  p->gridDimX * p->gridDimY * p->gridDimZ,
+				  p->blockDimX * p->blockDimY * p->blockDimZ,
+				  p->sharedMemBytes, p->hStream);
+			}
 			else if (cbinfo->callbackSite == CUPTI_API_EXIT)
+			{
 				Extrae_cudaLaunch_Exit();
+			}
 
 			ret = 1;
 		}
@@ -187,13 +194,19 @@ Extrae_RuntimeAPI_callback(CUpti_CallbackId cbid, const CUpti_CallbackData *cbin
 			  (cudaLaunch_v3020_params*)cbinfo->functionParams;
 
 			if (cbinfo->callbackSite == CUPTI_API_ENTER)
+			{
 # if CUPTI_API_VERSION >= 3
-				Extrae_cudaLaunch_Enter(p->func, NULL);
+				Extrae_cudaLaunch_Enter(p->func,
+				  NULL, NULL, NULL, NULL);
 # else
-				Extrae_cudaLaunch_Enter(p->entry, NULL);
+				Extrae_cudaLaunch_Enter(p->entry,
+				  NULL, NULL, NULL, NULL);
 # endif /* CUPTI_API_VERSION >= 3 */
+			}
 			else if (cbinfo->callbackSite == CUPTI_API_EXIT)
+			{
 				Extrae_cudaLaunch_Exit();
+			}
 
 			ret = 1;
 		}
@@ -594,9 +607,16 @@ Extrae_RuntimeAPI_callback(CUpti_CallbackId cbid, const CUpti_CallbackData *cbin
 			  (cudaLaunchKernel_v7000_params*)cbinfo->functionParams;
 
 			if (cbinfo->callbackSite == CUPTI_API_ENTER)
-				Extrae_cudaLaunch_Enter(p->func, p->stream);
+			{
+				Extrae_cudaLaunch_Enter(p->func, 
+				  p->gridDim.x * p->gridDim.y * p->gridDim.z,
+				  p->blockDim.x * p->blockDim.y * p->blockDim.z,
+				  p->sharedMem, p->stream);
+			}
 			else if (cbinfo->callbackSite == CUPTI_API_EXIT)
+			{
 				Extrae_cudaLaunch_Exit();
+			}
 
 			ret = 1;
 		}
@@ -609,9 +629,16 @@ Extrae_RuntimeAPI_callback(CUpti_CallbackId cbid, const CUpti_CallbackData *cbin
 			  (cudaLaunchKernel_ptsz_v7000_params*)cbinfo->functionParams;
 
 			if (cbinfo->callbackSite == CUPTI_API_ENTER)
-				Extrae_cudaLaunch_Enter(p->func, p->stream);
+			{
+				Extrae_cudaLaunch_Enter(p->func,
+				  p->gridDim.x * p->gridDim.y * p->gridDim.z,
+				  p->blockDim.x * p->blockDim.y * p->blockDim.z,
+				  p->sharedMem, p->stream);
+			}
 			else if (cbinfo->callbackSite == CUPTI_API_EXIT)
+			{
 				Extrae_cudaLaunch_Exit();
+			}
 
 			ret = 1;
 		}
@@ -626,9 +653,16 @@ Extrae_RuntimeAPI_callback(CUpti_CallbackId cbid, const CUpti_CallbackData *cbin
 			  (cudaLaunchCooperativeKernel_v9000_params*)cbinfo->functionParams;
 
 			if (cbinfo->callbackSite == CUPTI_API_ENTER)
-				Extrae_cudaLaunch_Enter(p->func, p->stream);
+			{
+				Extrae_cudaLaunch_Enter(p->func, 
+				  p->gridDim.x * p->gridDim.y * p->gridDim.z,
+				  p->blockDim.x * p->blockDim.y * p->blockDim.z,
+				  p->sharedMem, p->stream);
+			}
 			else if (cbinfo->callbackSite == CUPTI_API_EXIT)
+			{
 				Extrae_cudaLaunch_Exit();
+			}
 
 			ret = 1;
 		}
@@ -641,9 +675,16 @@ Extrae_RuntimeAPI_callback(CUpti_CallbackId cbid, const CUpti_CallbackData *cbin
 			  (cudaLaunchCooperativeKernel_ptsz_v9000_params*)cbinfo->functionParams;
 
 			if (cbinfo->callbackSite == CUPTI_API_ENTER)
-				Extrae_cudaLaunch_Enter(p->func, p->stream);
+			{
+				Extrae_cudaLaunch_Enter(p->func, 
+				  p->gridDim.x * p->gridDim.y * p->gridDim.z,
+				  p->blockDim.x * p->blockDim.y * p->blockDim.z,
+				  p->sharedMem, p->stream);
+			}
 			else if (cbinfo->callbackSite == CUPTI_API_EXIT)
+			{
 				Extrae_cudaLaunch_Exit();
+			}
 
 			ret = 1;
 		}
@@ -668,57 +709,65 @@ CUPTIAPI Extrae_CUPTI_callback(void *udata, CUpti_CallbackDomain domain,
 	UNREFERENCED_PARAMETER(domain);
 	int ret = 0;
 
-	if (!mpitrace_on || !Extrae_get_trace_CUDA() || cbinfo == NULL)
-		return;
-
-	/*
-	 * Filter out all nested callbacks
-	 */
-	if((cbinfo->callbackSite == CUPTI_API_ENTER))
-		Extrae_CUDA_updateDepth(cbinfo);
-
-	if ( (Extrae_CUDA_getDepth() <= 1) )
+	if( cbinfo != NULL )
 	{
-		switch (domain)
-		{
-			case CUPTI_CB_DOMAIN_DRIVER_API:
-				ret = Extrae_DriverAPI_callback(cbid, cbinfo);
-				break;
-			case CUPTI_CB_DOMAIN_RUNTIME_API:
-				ret = Extrae_RuntimeAPI_callback(cbid, cbinfo);
-				break;
-			default:
-				ret = -1;
-	#if LOG_OTHER_DOMAIN_UNTRACKED_CALLBACKS
-				fprintf(stderr, "CUDA call (domain = %d cbid = %d\n", domain, cbid);
-	#endif
-		}
+		if (cbinfo->callbackSite == CUPTI_API_ENTER) Backend_Enter_Instrumentation();
+	}
+	else
+	{
+		Backend_Enter_Instrumentation();
+	}
 
-		if (ret == 0)
+	if (EXTRAE_ON() && Extrae_get_trace_CUDA())
+	{
+		if( Backend_Get_InstrumentationLevel() == 1)
 		{
-			if (cbinfo->callbackSite == CUPTI_API_ENTER)
+			switch (domain)
 			{
-	#if LOG_UNTRACKED_CALLBACKS
-				const char *callbackName = "Untracked";
-				if (cuptiGetCallbackName_real != NULL)
-				{
-					cuptiGetCallbackName_real
-						(domain, cbid, &callbackName);
-				}
-				fprintf(stderr, "%s CUDA call (domain = %d cbid = %d)\n", callbackName, domain, cbid);
-	#endif
-
-				TRACE_EVENT(LAST_READ_TIME, CUDA_UNTRACKED_EV, cbid);
+				case CUPTI_CB_DOMAIN_DRIVER_API:
+					ret = Extrae_DriverAPI_callback(cbid, cbinfo);
+					break;
+				case CUPTI_CB_DOMAIN_RUNTIME_API:
+					ret = Extrae_RuntimeAPI_callback(cbid, cbinfo);
+					break;
+				default:
+					ret = -1;
+		#if LOG_OTHER_DOMAIN_UNTRACKED_CALLBACKS
+					fprintf(stderr, "CUDA call (domain = %d cbid = %d\n", domain, cbid);
+		#endif
 			}
-			else if (cbinfo->callbackSite == CUPTI_API_EXIT)
+
+			if (ret == 0)
 			{
-				TRACE_EVENT(TIME, CUDA_UNTRACKED_EV, EVT_END);
+				if (cbinfo->callbackSite == CUPTI_API_ENTER)
+				{
+		#if LOG_UNTRACKED_CALLBACKS
+					const char *callbackName = "Untracked";
+					if (cuptiGetCallbackName_real != NULL)
+					{
+						cuptiGetCallbackName_real
+							(domain, cbid, &callbackName);
+					}
+					fprintf(stderr, "%s CUDA call (domain = %d cbid = %d)\n", callbackName, domain, cbid);
+		#endif
+
+					TRACE_EVENT(LAST_READ_TIME, CUDA_UNTRACKED_EV, cbid);
+				}
+				else if (cbinfo->callbackSite == CUPTI_API_EXIT)
+				{
+					TRACE_EVENT(TIME, CUDA_UNTRACKED_EV, EVT_END);
+				}
 			}
 		}
 	}
-
-	if((cbinfo->callbackSite == CUPTI_API_EXIT))
-		Extrae_CUDA_updateDepth(cbinfo);
+	if( cbinfo != NULL )
+	{
+		if((cbinfo->callbackSite == CUPTI_API_EXIT)) Backend_Leave_Instrumentation();
+	}
+	else
+	{
+		Backend_Leave_Instrumentation();
+	}
 }
 
 void Extrae_CUDA_init (int rank)
@@ -766,6 +815,7 @@ cuptiGetCallbackName_real = (CUptiResult(*)(CUpti_CallbackDomain, uint32_t, cons
 	cuptiEnableCallback(0, subscriber, CUPTI_CB_DOMAIN_RUNTIME_API, CUPTI_RUNTIME_TRACE_CBID_cudaGLSetGLDevice_v3020); /* 63 */
 	cuptiEnableCallback(0, subscriber, CUPTI_CB_DOMAIN_RUNTIME_API, CUPTI_RUNTIME_TRACE_CBID_cudaGLSetBufferObjectMapFlags_v3020); /* 68 */
 	cuptiEnableCallback(0, subscriber, CUPTI_CB_DOMAIN_RUNTIME_API, CUPTI_RUNTIME_TRACE_CBID_cudaWGLGetDevice_v3020); /* 71 */
+	cuptiEnableCallback(0, subscriber, CUPTI_CB_DOMAIN_RUNTIME_API, CUPTI_RUNTIME_TRACE_CBID_cudaEventCreate_v3020); /* 133 */
 	cuptiEnableCallback(0, subscriber, CUPTI_CB_DOMAIN_RUNTIME_API, CUPTI_RUNTIME_TRACE_CBID_cudaEventCreateWithFlags_v3020); /* 134 */
 	cuptiEnableCallback(0, subscriber, CUPTI_CB_DOMAIN_RUNTIME_API, CUPTI_RUNTIME_TRACE_CBID_cudaEventDestroy_v3020); /* 136 */
 	cuptiEnableCallback(0, subscriber, CUPTI_CB_DOMAIN_RUNTIME_API, CUPTI_RUNTIME_TRACE_CBID_cudaEventElapsedTime_v3020); /* 139 */
@@ -775,4 +825,25 @@ cuptiGetCallbackName_real = (CUptiResult(*)(CUpti_CallbackDomain, uint32_t, cons
 	cuptiEnableCallback(0, subscriber, CUPTI_CB_DOMAIN_RUNTIME_API, CUPTI_RUNTIME_TRACE_CBID_cudaProfilerStop_v4000); /* 172 */
 	cuptiEnableCallback(0, subscriber, CUPTI_CB_DOMAIN_RUNTIME_API, CUPTI_RUNTIME_TRACE_CBID_cudaDeviceGetAttribute_v5000); /* 200 */
 	cuptiEnableCallback(0, subscriber, CUPTI_CB_DOMAIN_RUNTIME_API, CUPTI_RUNTIME_TRACE_CBID_cudaGetDeviceFlags_v7000); /* 212 */
+	cuptiEnableCallback(0, subscriber, CUPTI_CB_DOMAIN_DRIVER_API, CUPTI_DRIVER_TRACE_CBID_cuDriverGetVersion); /* 2 */
+	cuptiEnableCallback(0, subscriber, CUPTI_CB_DOMAIN_DRIVER_API, CUPTI_DRIVER_TRACE_CBID_cuDeviceGet); /* 3 */
+	cuptiEnableCallback(0, subscriber, CUPTI_CB_DOMAIN_DRIVER_API, CUPTI_DRIVER_TRACE_CBID_cuDeviceGetCount); /* 4 */
+	cuptiEnableCallback(0, subscriber, CUPTI_CB_DOMAIN_DRIVER_API, CUPTI_DRIVER_TRACE_CBID_cuDeviceGetName); /* 5 */
+	cuptiEnableCallback(0, subscriber, CUPTI_CB_DOMAIN_DRIVER_API, CUPTI_DRIVER_TRACE_CBID_cuDeviceGetAttribute); /* 9 */
+	cuptiEnableCallback(0, subscriber, CUPTI_CB_DOMAIN_DRIVER_API, CUPTI_DRIVER_TRACE_CBID_cuCtxGetDevice); /* 16 */
+	cuptiEnableCallback(0, subscriber, CUPTI_CB_DOMAIN_DRIVER_API, CUPTI_DRIVER_TRACE_CBID_cuEventCreate); /* 118 */
+	cuptiEnableCallback(0, subscriber, CUPTI_CB_DOMAIN_DRIVER_API, CUPTI_DRIVER_TRACE_CBID_cuEventElapsedTime); /* 123 */
+	cuptiEnableCallback(0, subscriber, CUPTI_CB_DOMAIN_DRIVER_API, CUPTI_DRIVER_TRACE_CBID_cuGetExportTable); /* 135 */
+	cuptiEnableCallback(0, subscriber, CUPTI_CB_DOMAIN_DRIVER_API, CUPTI_DRIVER_TRACE_CBID_cuCtxGetLimit); /* 137 */
+	cuptiEnableCallback(0, subscriber, CUPTI_CB_DOMAIN_DRIVER_API, CUPTI_DRIVER_TRACE_CBID_cuDeviceTotalMem_v2); /* 259 */
+	cuptiEnableCallback(0, subscriber, CUPTI_CB_DOMAIN_DRIVER_API, CUPTI_DRIVER_TRACE_CBID_cuCtxSetCurrent); /* 303 */
+	cuptiEnableCallback(0, subscriber, CUPTI_CB_DOMAIN_DRIVER_API, CUPTI_DRIVER_TRACE_CBID_cuCtxGetCurrent); /* 304 */
+	cuptiEnableCallback(0, subscriber, CUPTI_CB_DOMAIN_DRIVER_API, CUPTI_DRIVER_TRACE_CBID_cuEventDestroy_v2); /* 325 */
+	cuptiEnableCallback(0, subscriber, CUPTI_CB_DOMAIN_DRIVER_API, CUPTI_DRIVER_TRACE_CBID_cuDevicePrimaryCtxRetain); /* 386 */
+	cuptiEnableCallback(0, subscriber, CUPTI_CB_DOMAIN_DRIVER_API, CUPTI_DRIVER_TRACE_CBID_cuDeviceGetUuid); /* 482 */
+	cuptiEnableCallback(0, subscriber, CUPTI_CB_DOMAIN_DRIVER_API, CUPTI_DRIVER_TRACE_CBID_cuModuleGetLoadingMode); /* 673 */
+	cuptiEnableCallback(0, subscriber, CUPTI_CB_DOMAIN_DRIVER_API, CUPTI_DRIVER_TRACE_CBID_cuStreamBatchMemOp_v2); /* 667 */
+	cuptiEnableCallback(0, subscriber, CUPTI_CB_DOMAIN_DRIVER_API, CUPTI_DRIVER_TRACE_CBID_cuGetProcAddress_v2); /* 677 */
+	cuptiEnableCallback(0, subscriber, CUPTI_CB_DOMAIN_DRIVER_API, CUPTI_DRIVER_TRACE_CBID_cuLibraryLoadData); /* 678 */
+	cuptiEnableCallback(0, subscriber, CUPTI_CB_DOMAIN_DRIVER_API, CUPTI_DRIVER_TRACE_CBID_cuLibraryGetModule); /* 682 */
 }
