@@ -26,7 +26,6 @@
 
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
-# include <stdint.h>
 #endif
 #ifdef HAVE_STDLIB_H
 # include <stdlib.h>
@@ -34,12 +33,11 @@
 
 #include "hip_common.h"
 
-#include <pthread.h>
-
 #include "taskid.h"
 #include "threadinfo.h"
 #include "wrapper.h"
 #include "trace_macros.h"
+#include "trace_macros_gpu.h"
 #include "hip_probe.h"
 #include "xalloc.h"
 #include "extrae_user_events.h"
@@ -61,10 +59,15 @@
 #ifndef HIP_MAP_NUM_BUCKETS
 #define HIP_MAP_NUM_BUCKETS 4096u
 #endif
- 
+
+/* Structures that will hold the parameters needed for the exit parts of the
+ * instrumentation code. This way we can support dyninst/ld-preload/cupti
+ * instrumentation with a single file
+ */
+
 typedef struct 
 {
-	hipStream_t* stream;
+	hipStream_t *stream;
 } hipStreamCreate_saved_params_t;
 
 typedef struct
@@ -91,23 +94,37 @@ typedef union
 	hipStreamSynchronize_saved_params_t css;
 	hipMemcpy_saved_params_t cm;
 	hipMemcpyAsync_saved_params_t cma;
-} Extrae_HIP_saved_params_union;
+} Extrae_hip_saved_params_union;
 
 typedef struct
 {
 	int instrumentationDepth;
-	Extrae_HIP_saved_params_union punion;
-} Extrae_HIP_saved_params_t;
+	Extrae_hip_saved_params_union punion;
+} Extrae_hip_saved_params_t;
 
 unsigned hip_events_block_size = DEFAULT_HIP_EVENTS_BLOCK_SIZE;
 
-static Extrae_HIP_saved_params_t* Extrae_HIP_saved_params = NULL;
+static Extrae_hip_saved_params_t *Extrae_HIP_saved_params = NULL;
 
 static unsigned __last_tag = 0xC0DA; /* Fixed tag */
+static unsigned Extrae_HIP_tag_generator(void)
+{
+	__last_tag++;
+	return __last_tag;
+}
 
-static struct HIPdevices_t* devices = NULL;
+static unsigned Extrae_HIP_tag_get()
+{
+	return __last_tag;
+}
+
+static struct HIPdevices_t *devices = NULL;
 static int HIPdevices = 0;
 
+/**
+ * Variable to control the initialization and allow the flushing of
+ * events at the exit point.
+ */
 int hipInitialized = 0;
 
 gpu_event_list_t availableEvents;
@@ -128,17 +145,6 @@ int Extrae_HIP_getDepth()
 	return Extrae_HIP_saved_params[THREADID].instrumentationDepth;
 }
 
-static unsigned Extrae_HIP_tag_generator (void)
-{
-	__last_tag++;
-	return __last_tag;
-}
-
-static unsigned Extrae_HIP_tag_get()
-{
-	return __last_tag;
-}
-
 const char* NormalizeKernelName(const char* name)
 {
     static char buffer[512];
@@ -153,7 +159,7 @@ const char* NormalizeKernelName(const char* name)
     return name;
 }
 
-static void Extrae_HIP_SynchronizeStream (int devid, int streamid)
+static void Extrae_HIP_SynchronizeStream(int devid, int streamid)
 {
 	int err;
 
@@ -169,7 +175,7 @@ static void Extrae_HIP_SynchronizeStream (int devid, int streamid)
 
 	if(devices[devid].Stream[streamid].device_reference_event == NULL)
 	{
-		devices[devid].Stream[streamid].device_reference_event = gpuEventList_pop(&availableEvents);
+		devices[devid].Stream[streamid].device_reference_event = gpuEventList_pop(&devices[devid].availableEvents);
 	}
 
 	err = hipEventRecord (devices[devid].Stream[streamid].device_reference_event->ts_event, devices[devid].Stream[streamid].stream);
@@ -181,7 +187,20 @@ static void Extrae_HIP_SynchronizeStream (int devid, int streamid)
 	devices[devid].Stream[streamid].host_reference_time = TIME;
 }
 
-void Extrae_HIP_Initialize (int devid)
+void Extrae_HIP_deInitialize(int devid)
+{
+	if (devices != NULL)
+	{
+		if (devices[devid].initialized)
+		{
+			xfree (devices[devid].Stream);
+			devices[devid].Stream = NULL;
+			devices[devid].initialized = FALSE;
+		}
+	}
+}
+
+void Extrae_HIP_Initialize(int devid)
 {
 	hipError_t err;
 	int i;
@@ -241,10 +260,10 @@ void Extrae_HIP_Initialize (int devid)
 		/* default device stream */
 		devices[devid].Stream[0].stream = (hipStream_t) 0;
 		gpuEventList_init(&devices[devid].Stream[0].gpu_event_list, FALSE, XTR_HIP_EVENTS_BLOCK_SIZE);
-		gpuEventList_init(&availableEvents, TRUE, XTR_HIP_EVENTS_BLOCK_SIZE);
+		gpuEventList_init(&devices[devid].availableEvents, TRUE, XTR_HIP_EVENTS_BLOCK_SIZE);
 
 		/* Create an event record and process it through the stream! */
-		/* FIX HI_EVENT_BLOCKING_SYNC may be harmful!? */
+		/* FIX CU_EVENT_BLOCKING_SYNC may be harmful!? */
 
 		devices[devid].Stream[0].device_reference_event = NULL;
 		Extrae_HIP_SynchronizeStream (devid, 0);
@@ -270,24 +289,11 @@ void Extrae_HIP_Initialize (int devid)
 	}
 }
 
-void Extrae_HIP_deInitialize (int devid)
-{
-	if (devices != NULL)
-	{
-		if (devices[devid].initialized)
-		{
-			xfree (devices[devid].Stream);
-			devices[devid].Stream = NULL;
-			devices[devid].initialized = FALSE;
-		}
-	}
-}
-
-static int Extrae_HIP_SearchStream (int devid, hipStream_t stream)
+static int Extrae_HIP_SearchStream(int devid, hipStream_t stream)
 {
 	int i;
 
-	/* Starting from HIP 8, HI_STREAM_LEGACY is a new stream handle that uses 
+	/* Starting from HIP 8, CU_STREAM_LEGACY is a new stream handle that uses 
 	   an implicit stream with legacy synchronization behavior, just as the 
 	   behaviour of stream 0 (default).
 
@@ -304,7 +310,30 @@ static int Extrae_HIP_SearchStream (int devid, hipStream_t stream)
 	return -1;
 }
 
-static void Extrae_HIP_RegisterStream (int devid, hipStream_t stream)
+static void Extrae_HIP_unRegisterStream(int devid, hipStream_t stream)
+{
+	int stid = Extrae_HIP_SearchStream (devid, stream);
+
+#ifdef DEBUG
+	fprintf (stderr, "Extrae_HIP_unRegisterStream (devid=%d, stream=%p unassigned from streamid => %d/%d\n", devid, stream, stid, devices[devid].nstreams);
+#endif
+
+	Extrae_HIP_flush_streams(devid, stid);
+
+	int nstreams = devices[devid].nstreams - 1;
+
+	struct RegisteredStreams_t *rs_tmp = (struct RegisteredStreams_t*) xmalloc (nstreams*sizeof(struct RegisteredStreams_t));
+
+	memmove (rs_tmp, devices[devid].Stream, stid * sizeof(struct RegisteredStreams_t));
+	memmove (rs_tmp+stid, devices[devid].Stream + stid + 1, (devices[devid].nstreams - stid - 1)*sizeof(struct RegisteredStreams_t));
+
+	devices[devid].nstreams = nstreams;
+
+	xfree (devices[devid].Stream);
+	devices[devid].Stream = rs_tmp;
+}
+
+static void Extrae_HIP_RegisterStream(int devid, hipStream_t stream)
 {
 	int i = devices[devid].nstreams;
 
@@ -356,93 +385,96 @@ static void Extrae_HIP_RegisterStream (int devid, hipStream_t stream)
 	THREAD_TRACE_MISCEVENT(devices[devid].Stream[i].threadid, devices[devid].Stream[i].host_reference_time, TRACING_MODE_EV, TRACE_MODE_DETAIL, 0);
 }
 
-static void Extrae_HIP_unRegisterStream (int devid, hipStream_t stream)
-{
-	int stid = Extrae_HIP_SearchStream (devid, stream);
-
-#ifdef DEBUG
-	fprintf (stderr, "Extrae_HIP_unRegisterStream (devid=%d, stream=%p unassigned from streamid => %d/%d\n", devid, stream, stid, devices[devid].nstreams);
-#endif
-
-	Extrae_HIP_flush_streams(devid, stid);
-
-	int nstreams = devices[devid].nstreams - 1;
-
-	struct RegisteredStreams_t* rs_tmp = (struct RegisteredStreams_t*) xmalloc (nstreams*sizeof(struct RegisteredStreams_t));
-
-	memmove (rs_tmp, devices[devid].Stream, stid * sizeof(struct RegisteredStreams_t));
-	memmove (rs_tmp+stid, devices[devid].Stream + stid + 1, (devices[devid].nstreams - stid - 1)*sizeof(struct RegisteredStreams_t));
-
-	devices[devid].nstreams = nstreams;
-
-	xfree (devices[devid].Stream);
-	devices[devid].Stream = rs_tmp;
-}
-
-static void Extrae_HIP_AddEventToStream (Extrae_HIP_Time_Type timetype, int devid, int streamid, unsigned event, unsigned long long value, unsigned tag, unsigned size)
+static void Extrae_HIP_AddEventToStream(Extrae_HIP_Time_Type timetype,
+	int devid, 
+	int streamid, 
+	unsigned event, 
+	unsigned long long value,
+	unsigned tag, 
+	size_t size, 
+	int blockspergridX,
+	int blockspergridY,
+	int blockspergridZ, 
+	int threadsperblockX,
+	int threadsperblockY,
+	int threadsperblockZ)
 {
 	int err;
-	struct RegisteredStreams_t* registered_stream = &devices[devid].Stream[streamid];
+	struct RegisteredStreams_t *registered_stream = &devices[devid].Stream[streamid];
 
-	gpu_event_t* gpu_event = gpuEventList_pop(&availableEvents);
+	gpu_event_t* gpu_event = gpuEventList_pop(&devices[devid].availableEvents);
 
+	dim3 blockspergrid = {blockspergridX, blockspergridY, blockspergridZ};
+	dim3 threadsperblock = {threadsperblockX, threadsperblockY, threadsperblockZ};
+
+	gpu_event->timetype = timetype;
 	gpu_event->event = event;
 	gpu_event->value = value;
 	gpu_event->tag = tag;
-	gpu_event->size = size;
-	gpu_event->timetype = timetype;
+	gpu_event->memSize = size;
+	gpu_event->blocksPerGrid = blockspergrid;
+	gpu_event->threadsPerBlock = threadsperblock;
 
-	err = hipEventRecord (gpu_event->ts_event, registered_stream->stream);
+	err = hipEventRecord(gpu_event->ts_event, registered_stream->stream);
 	CHECK_HI_ERROR(err, hipEventRecord);
 
 	gpuEventList_add(&registered_stream->gpu_event_list, gpu_event);
 }
 
-static void traceGPUEvents(int threadid, UINT64 time, unsigned event, unsigned long long value, unsigned size, unsigned tag)
+/**
+ * This routine emits GPU events into the tracing buffer and
+ * inserts communication events to connect host-side 
+ * HIP runtime calls with their corresponding GPU kernel and memcopies executions.
+ *
+ * @param threadid          buffer thread id
+ * @param time              Timestamp of the event
+ * @param event             kind of event (HIPKERNEL_GPU_VAL, HIPMEMCPY_GPU_VAL...)
+ * @param value             Entry/exit (EVT_BEGIN/EVT_END)
+ * @param tag               Communication tag
+ * @param size              Size in bytes for memory operations
+ * @param blockspergrid     Grid configuration (HIP kernels)
+ * @param threadsperblock   Block configuration (HIP kernels)
+ */
+static void traceGPUEvents(int threadid, 
+	UINT64 time, 
+	unsigned event, 
+	unsigned long long value, 
+	unsigned tag, 
+	size_t size, 
+	unsigned blockspergrid, 
+	unsigned threadsperblock)
 {
-	/* Emit events into the tracing buffer. */
-	THREAD_TRACE_MISCEVENT (threadid, time, HIPCALLGPU_EV, event, value);
+	TRACE_GPU_EVENT(threadid, time, HIPCALLGPU_EV, event, value, size);
 
 	if (event == HIPKERNEL_GPU_VAL)
 	{
-		THREAD_TRACE_MISCEVENT(threadid, time, HIPFUNC_EV, value, 0);
-	}
-
-	/* Emit communication records for memory transfer, kernel setup and kernel execution */
-	if (event == HIPMEMCPY_GPU_VAL || event == HIPMEMCPYASYNC_GPU_VAL)
-	{
-		if (value != EVT_END)
+		TRACE_GPU_KERNEL_EVENT(threadid, time, HIP_KERNEL_EXEC_EV, value, blockspergrid, threadsperblock);
+		if(value != EVT_END) 
 		{
-			THREAD_TRACE_MISCEVENT(threadid, time, HIP_DYNAMIC_MEM_SIZE_EV, size, 0);
+			THREAD_TRACE_USER_COMMUNICATION_EVENT(threadid, time,USER_RECV_EV,TASKID,0, tag, tag);
 		}
-
-		if (tag > 0)
+	}
+	else
+	{
+		if ((event == HIPMEMCPY_GPU_VAL || event == HIPMEMCPYASYNC_GPU_VAL) && (tag > 0))
 		{
 			THREAD_TRACE_USER_COMMUNICATION_EVENT(threadid, time, (value==EVT_END)?USER_RECV_EV:USER_SEND_EV, TASKID, size, tag, tag);
 		}
 	}
-	else if (event == HIPKERNEL_GPU_VAL && value != EVT_END)
-	{
-		THREAD_TRACE_USER_COMMUNICATION_EVENT(threadid, time,USER_RECV_EV,TASKID,0, tag, tag);
-	}
-	else if (event == HIPCONFIGKERNEL_GPU_VAL && value != EVT_END)
-	{
-		THREAD_TRACE_USER_COMMUNICATION_EVENT(threadid, time, USER_RECV_EV, TASKID, 0, tag, tag);
-	}
 }
 
-static void flushGPUEvents (int devid, int streamid)
+static void flushGPUEvents(int devid, int streamid)
 {
 	int threadid = devices[devid].Stream[streamid].threadid;
 	int err;
 	UINT64 utmp, last_time = 0;
 	float ftmp;
-	gpu_event_t* gpu_event;
-	hipEvent_t* reference_event;
+	gpu_event_t *gpu_event;
+	hipEvent_t *reference_event;
 	UINT64 reference_time = 0;
 	struct RegisteredStreams_t *registered_stream = &devices[devid].Stream[streamid];
 
-	gpu_event_t* last_gpu_event = gpuEventList_peek_tail(&registered_stream->gpu_event_list);
+	gpu_event_t *last_gpu_event = gpuEventList_peek_tail(&registered_stream->gpu_event_list);
 
 	if(last_gpu_event != NULL)
 	{
@@ -473,20 +505,27 @@ static void flushGPUEvents (int devid, int streamid)
 				utmp = last_time;
 			}
 
-			traceGPUEvents(threadid, utmp, gpu_event->event, gpu_event->value, gpu_event->size, gpu_event->tag);
+			traceGPUEvents(threadid, 
+				utmp, 
+				gpu_event->event, 
+				gpu_event->value, 
+				gpu_event->tag, 
+				gpu_event->memSize, 
+				0, 
+				0);
 
 			last_time = utmp;
 
 			if(gpuEventList_isempty(&registered_stream->gpu_event_list))
 			{
-				gpuEventList_add(&availableEvents, registered_stream->device_reference_event);
+				gpuEventList_add(&devices[devid].availableEvents, registered_stream->device_reference_event);
 				registered_stream->device_reference_event = gpu_event;
 				registered_stream->host_reference_time = last_time;
 				gpu_event = NULL;
 			}
 			else
 			{
-				gpuEventList_add(&availableEvents, gpu_event);
+				gpuEventList_add(&devices[devid].availableEvents, gpu_event);
 				gpu_event = gpuEventList_pop(&registered_stream->gpu_event_list);
 			}
 		}
@@ -640,13 +679,12 @@ void hipmap_clear_all_safe(void)
 }
 
 /****************************************************************************/
-/* HIP INSTRUMENTATION                                                      */
+/* HIP INSTRUMENTATION                                                     */
 /****************************************************************************/
 
 __thread int _hipLaunch_stream = 0;
-__thread int _hipLaunch_devid  = -1;
 
-void Extrae_hipConfigureCall_Enter(dim3 p1, dim3 p2, size_t p3, hipStream_t p4)
+void Extrae_hipConfigureCall_Enter (dim3 p1, dim3 p2, size_t p3, hipStream_t p4)
 {
 	int strid;
 	int devid;
@@ -659,8 +697,7 @@ void Extrae_hipConfigureCall_Enter(dim3 p1, dim3 p2, size_t p3, hipStream_t p4)
 	hipGetDevice (&devid);
 	Extrae_HIP_Initialize (devid);
 
-	Backend_Enter_Instrumentation ();
-	Probe_HIP_ConfigureCall_Entry ();
+	Probe_Hip_ConfigureCall_Entry ();
 
 	TRACE_USER_COMMUNICATION_EVENT (LAST_READ_TIME, USER_SEND_EV, TASKID, 0, tag, tag);
 
@@ -671,61 +708,46 @@ void Extrae_hipConfigureCall_Enter(dim3 p1, dim3 p2, size_t p3, hipStream_t p4)
 		exit (-1);
 	}
 	_hipLaunch_stream = strid;
-	Extrae_HIP_AddEventToStream(EXTRAE_HIP_NEW_TIME, devid, strid, HIPCONFIGKERNEL_GPU_VAL, EVT_BEGIN, tag, tag);
+
+	Extrae_HIP_AddEventToStream(EXTRAE_HIP_NEW_TIME, 
+		devid, 
+		strid,
+		HIPCONFIGKERNEL_GPU_VAL, 
+		EVT_BEGIN, 
+		tag, 
+		tag, 
+		0,0,0,
+		0,0,0);
 }
 
-void Extrae_hipConfigureCall_Exit(void)
+void Extrae_hipConfigureCall_Exit (void)
 {
 	int devid;
 
 	hipGetDevice (&devid);
 
-	Extrae_HIP_AddEventToStream(EXTRAE_HIP_NEW_TIME, devid, _hipLaunch_stream, HIPCONFIGKERNEL_GPU_VAL, EVT_END, Extrae_HIP_tag_get(), 0);
-
-	Probe_HIP_ConfigureCall_Exit ();
-	Backend_Leave_Instrumentation ();
-}
-
-void Extrae_hipLaunchKernel_Enter(const char* p1, hipStream_t stream)
-{
-	int devid;
-	unsigned tag = Extrae_HIP_tag_generator();
-
-	hipGetDevice(&devid);
-	Extrae_HIP_Initialize(devid);
-
-	Backend_Enter_Instrumentation();
-	Probe_HIP_Launch_Entry ((UINT64) p1);
-
-	TRACE_USER_COMMUNICATION_EVENT (LAST_READ_TIME, USER_SEND_EV, TASKID, 0, tag, tag);
-
-	int strid = Extrae_HIP_SearchStream (devid, stream);
-	_hipLaunch_stream = strid;
-
 	Extrae_HIP_AddEventToStream(EXTRAE_HIP_NEW_TIME, 
-								devid, 
-								_hipLaunch_stream, 
-								HIPKERNEL_GPU_VAL, 
-								(UINT64)p1, 
-								tag, 
-								tag);
+		devid, 
+		_hipLaunch_stream, 
+		HIPCONFIGKERNEL_GPU_VAL, 
+		EVT_END, 
+		Extrae_HIP_tag_get(), 
+		0,
+		0,0,0,
+		0,0,0);
+
+	Probe_Hip_ConfigureCall_Exit ();
 }
 
-void Extrae_hipLaunchKernel_Exit(void)
-{
-	int devid;
-
-	hipGetDevice(&devid);
-	Extrae_HIP_Initialize(devid);
-
-	Extrae_HIP_AddEventToStream(EXTRAE_HIP_NEW_TIME, devid, _hipLaunch_stream, 
-		HIPKERNEL_GPU_VAL, EVT_END, Extrae_HIP_tag_get(), 0);
-
-	Probe_HIP_Launch_Exit ();
-	Backend_Leave_Instrumentation();
-}
-
-void Extrae_hipModuleLaunchKernel_Enter(hipFunction_t f, hipStream_t stream)
+void Extrae_hipLaunchKernel_Enter(const char* address, 
+	unsigned int gridDimX, 
+	unsigned int gridDimY, 
+	unsigned int gridDimZ, 
+	unsigned int blockDimX, 
+	unsigned int blockDimY, 
+	unsigned int blockDimZ, 
+	size_t sharedMemBytes, 
+	hipStream_t stream)
 {
 	int devid;
 	unsigned tag = Extrae_HIP_tag_generator();
@@ -733,71 +755,128 @@ void Extrae_hipModuleLaunchKernel_Enter(hipFunction_t f, hipStream_t stream)
 	hipGetDevice(&devid);
 	Extrae_HIP_Initialize(devid);
 
-	Backend_Enter_Instrumentation();
-	Probe_HIP_Launch_Entry((UINT64)f);
+	Probe_Hip_Launch_Entry((UINT64) address);
 
 	TRACE_USER_COMMUNICATION_EVENT(LAST_READ_TIME, USER_SEND_EV, TASKID, 0, tag, tag);
 
 	int strid = Extrae_HIP_SearchStream(devid, stream);
 	_hipLaunch_stream = strid;
 
-	Extrae_HIP_AddEventToStream(EXTRAE_HIP_NEW_TIME,
-								devid, 
-								_hipLaunch_stream,
-								HIPKERNEL_GPU_VAL,
-								(UINT64)f,
-								tag, 
-								tag);
+	Extrae_HIP_AddEventToStream(EXTRAE_HIP_NEW_TIME, 
+		devid, 
+		_hipLaunch_stream, 
+		HIPKERNEL_GPU_VAL, 
+		(UINT64)address, 
+		tag,
+		sharedMemBytes,
+		gridDimX,gridDimY,gridDimZ, 
+		blockDimX,blockDimY,blockDimZ);
+
+}
+
+void Extrae_hipLaunchKernel_Exit(void)
+{
+	int devid;
+
+	hipGetDevice (&devid);
+	Extrae_HIP_Initialize (devid);
+
+	Extrae_HIP_AddEventToStream(EXTRAE_HIP_NEW_TIME, 
+		devid, 
+		_hipLaunch_stream,
+		HIPKERNEL_GPU_VAL, 
+		EVT_END, 
+		Extrae_HIP_tag_get(), 
+		0, 
+		0,0,0, 
+		0,0,0);
+
+	Probe_Hip_Launch_Exit ();
+}
+
+void Extrae_hipModuleLaunchKernel_Enter(hipFunction_t f, 
+	unsigned int gridDimX, 
+	unsigned int gridDimY, 
+	unsigned int gridDimZ, 
+	unsigned int blockDimX, 
+	unsigned int blockDimY, 
+	unsigned int blockDimZ, 
+	size_t sharedMemBytes, 
+	hipStream_t stream)
+{
+	int devid;
+	unsigned tag = Extrae_HIP_tag_generator();
+
+	hipGetDevice (&devid);
+	Extrae_HIP_Initialize (devid);
+
+	Probe_Hip_Launch_Entry((UINT64) f);
+
+	TRACE_USER_COMMUNICATION_EVENT(LAST_READ_TIME, USER_SEND_EV, TASKID, 0, tag, tag);
+
+	int strid = Extrae_HIP_SearchStream(devid, stream);
+	_hipLaunch_stream = strid;
+
+	Extrae_HIP_AddEventToStream(EXTRAE_HIP_NEW_TIME, 
+		devid, 
+		_hipLaunch_stream,
+		HIPKERNEL_GPU_VAL, 
+		(UINT64)f, 
+		tag, 
+		sharedMemBytes, 
+		gridDimX,gridDimY,gridDimZ, 
+		blockDimX,blockDimY,blockDimZ); 
+
 }
 
 void Extrae_hipModuleLaunchKernel_Exit(void)
 {
-    int devid = _hipLaunch_devid;
-    if (devid < 0) hipGetDevice(&devid);
+	int devid;
 
-    Extrae_HIP_Initialize(devid);
+	hipGetDevice (&devid);
+	Extrae_HIP_Initialize (devid);
 
-    Extrae_HIP_AddEventToStream(EXTRAE_HIP_NEW_TIME,
-                                devid, _hipLaunch_stream,
-                                HIPKERNEL_GPU_VAL,
-                                EVT_END,
-                                Extrae_HIP_tag_get(), 0);
+	Extrae_HIP_AddEventToStream(EXTRAE_HIP_NEW_TIME, 
+		devid, 
+		_hipLaunch_stream, 
+		HIPKERNEL_GPU_VAL, 
+		EVT_END, 
+		Extrae_HIP_tag_get(), 
+		0, 
+		0,0,0, 
+		0,0,0);
 
-    Probe_HIP_Launch_Exit();
-    Backend_Leave_Instrumentation();
+	Probe_Hip_Launch_Exit ();
 }
 
-void Extrae_hipMalloc_Enter(unsigned int event, void** devPtr, size_t size)
+void Extrae_hipMalloc_Enter(unsigned int event, void **devPtr, size_t size)
 {
 	int devid;
 
 	hipGetDevice(&devid);
 	Extrae_HIP_Initialize(devid);
 
-	Backend_Enter_Instrumentation();
-	Probe_HIP_Malloc_Entry(event, (UINT64)devPtr, size);
+	Probe_Hip_Malloc_Entry(event, (UINT64)devPtr, size);
 }
 
 void Extrae_hipMalloc_Exit(unsigned int event)
-{	
-	int devid;
-
-	hipGetDevice(&devid);
-	Extrae_HIP_Initialize(devid);
-
-	Probe_HIP_Malloc_Exit(event);
-	Backend_Leave_Instrumentation();
-}
-
-void Extrae_hipFree_Enter(unsigned int event, void* devPtr)
 {
 	int devid;
 
 	hipGetDevice(&devid);
 	Extrae_HIP_Initialize(devid);
 
-	Backend_Enter_Instrumentation();
-	Probe_HIP_Free_Entry(event, (UINT64)devPtr);
+	Probe_Hip_Malloc_Exit(event);
+}
+
+void Extrae_hipFree_Enter(unsigned int event, void *devPtr)
+{
+	int devid;
+
+	hipGetDevice(&devid);
+	Extrae_HIP_Initialize(devid);
+
+	Probe_Hip_Free_Entry(event, (UINT64)devPtr);
 }
 
 void Extrae_hipFree_Exit(unsigned int event)
@@ -807,31 +886,27 @@ void Extrae_hipFree_Exit(unsigned int event)
 	hipGetDevice(&devid);
 	Extrae_HIP_Initialize(devid);
 
-	Probe_HIP_Free_Exit(event);
-	Backend_Leave_Instrumentation();
+	Probe_Hip_Free_Exit(event);
 }
 
-void Extrae_hipHostAlloc_Enter(void** pHost, size_t size)
+void Extrae_hipHostAlloc_Enter(void **pHost, size_t size)
 {
-	Backend_Enter_Instrumentation();
-	Probe_HIP_HostAlloc_Entry((UINT64)pHost, size);
+	Probe_Hip_HostAlloc_Entry((UINT64)pHost, size);
 }
 
 void Extrae_hipHostAlloc_Exit(void)
 {
-	Probe_HIP_HostAlloc_Exit();
-	Backend_Leave_Instrumentation();
+	Probe_Hip_HostAlloc_Exit();
 }
 
 void Extrae_hipDeviceSynchronize_Enter(void)
 {
 	int devid;
 
-	hipGetDevice(&devid);
-	Extrae_HIP_Initialize(devid);
+	hipGetDevice (&devid);
+	Extrae_HIP_Initialize (devid);
 
-	Backend_Enter_Instrumentation();
-	Probe_HIP_ThreadBarrier_Entry();
+	Probe_Hip_ThreadBarrier_Entry ();
 	Extrae_HIP_flush_streams(devid, XTR_FLUSH_ALL_STREAMS);
 }
 
@@ -841,55 +916,30 @@ void Extrae_hipDeviceSynchronize_Exit(void)
 
 	hipGetDevice (&devid);
 
-	Probe_HIP_ThreadBarrier_Exit ();
-	Backend_Leave_Instrumentation ();
+	Probe_Hip_ThreadBarrier_Exit ();
 }
 
 void Extrae_hipThreadSynchronize_Enter(void)
 {
 	int devid;
 
-	hipGetDevice(&devid);
-	Extrae_HIP_Initialize(devid);
+	hipGetDevice (&devid);
+	Extrae_HIP_Initialize (devid);
 
-	Backend_Enter_Instrumentation();
-	Probe_HIP_ThreadBarrier_Entry();
+	Probe_Hip_ThreadBarrier_Entry ();
+	Extrae_HIP_flush_streams(devid, XTR_FLUSH_ALL_STREAMS);
 }
 
 void Extrae_hipThreadSynchronize_Exit(void)
 {
 	int devid;
 
-	hipGetDevice(&devid);
+	hipGetDevice (&devid);
 
-	Probe_HIP_ThreadBarrier_Exit();
-	Extrae_HIP_flush_streams(devid, XTR_FLUSH_ALL_STREAMS);
-	Backend_Leave_Instrumentation();
+	Probe_Hip_ThreadBarrier_Exit ();
 }
 
-void Extrae_hipModuleGetFunction_Enter(void)
-{
-	int devid;
-
-	hipGetDevice(&devid);
-	Extrae_HIP_Initialize(devid);
-
-	Backend_Enter_Instrumentation();
-	Probe_HIP_ThreadBarrier_Entry();
-}
-
-void Extrae_hipModuleGetFunction_Exit(void)
-{
-	int devid;
-
-	hipGetDevice(&devid);
-	Extrae_HIP_Initialize(devid);
-
-	Backend_Enter_Instrumentation();
-	Probe_HIP_ThreadBarrier_Entry();
-}
-
-void Extrae_HIP_flush_streams(int device_id, int stream_id )
+void Extrae_HIP_flush_streams(int device_id, int stream_id)
 {
 	int d = 0, s = 0;
 
@@ -914,14 +964,13 @@ void Extrae_HIP_flush_streams(int device_id, int stream_id )
 	}
 }
 
-void Extrae_hipStreamCreate_Enter(hipStream_t* p1)
+void Extrae_hipStreamCreate_Enter(hipStream_t *p1)
 {
 	ASSERT(Extrae_HIP_saved_params!=NULL, "Unallocated Extrae_HIP_saved_params");
 
 	Extrae_HIP_saved_params[THREADID].punion.csc.stream = p1;
 
-	Backend_Enter_Instrumentation ();
-	Probe_HIP_StreamCreate_Entry ();
+	Probe_Hip_StreamCreate_Entry();
 }
 
 void Extrae_hipStreamCreate_Exit(void)
@@ -931,13 +980,12 @@ void Extrae_hipStreamCreate_Exit(void)
 	ASSERT(Extrae_HIP_saved_params!=NULL, "Unallocated Extrae_HIP_saved_params");
 
 	hipGetDevice (&devid);
-	Extrae_HIP_Initialize (devid);
+	Extrae_HIP_Initialize(devid);
 
-	Extrae_HIP_RegisterStream (devid,
+	Extrae_HIP_RegisterStream(devid,
 	  *Extrae_HIP_saved_params[THREADID].punion.csc.stream);
 
-	Probe_HIP_StreamCreate_Exit ();
-	Backend_Leave_Instrumentation ();
+	Probe_Hip_StreamCreate_Exit();
 }
 
 void Extrae_hipStreamDestroy_Enter(hipStream_t stream)
@@ -946,13 +994,12 @@ void Extrae_hipStreamDestroy_Enter(hipStream_t stream)
 
 	ASSERT(Extrae_HIP_saved_params!=NULL, "Unallocated Extrae_HIP_saved_params");
 
-	Backend_Enter_Instrumentation ();
-	Probe_HIP_StreamDestroy_Entry ();
+	Probe_Hip_StreamDestroy_Entry();
 
 	Extrae_HIP_saved_params[THREADID].punion.css.stream = stream;
 
-	hipGetDevice (&devid);
-	Extrae_HIP_Initialize (devid);
+	hipGetDevice(&devid);
+	Extrae_HIP_Initialize(devid);
 
 	Extrae_HIP_unRegisterStream(devid, Extrae_HIP_saved_params[THREADID].punion.css.stream);
 }
@@ -961,8 +1008,7 @@ void Extrae_hipStreamDestroy_Exit(void)
 {
 	ASSERT(Extrae_HIP_saved_params!=NULL, "Unallocated Extrae_HIP_saved_params");
 
-	Probe_HIP_StreamDestroy_Exit ();
-	Backend_Leave_Instrumentation ();
+	Probe_Hip_StreamDestroy_Exit();
 }
 
 void Extrae_hipStreamSynchronize_Enter(hipStream_t p1)
@@ -974,14 +1020,13 @@ void Extrae_hipStreamSynchronize_Enter(hipStream_t p1)
 
 	Extrae_HIP_saved_params[THREADID].punion.css.stream = p1;
 
-	hipGetDevice(&devid);
+	hipGetDevice (&devid);
 	Extrae_HIP_Initialize (devid);
 
 	strid = Extrae_HIP_SearchStream (devid,
 	  Extrae_HIP_saved_params[THREADID].punion.css.stream);
 
-	Backend_Enter_Instrumentation ();
-	Probe_HIP_StreamBarrier_Entry (devices[devid].Stream[strid].threadid);
+	Probe_Hip_StreamBarrier_Entry (devices[devid].Stream[strid].threadid);
 
 	Extrae_HIP_flush_streams(devid, strid);
 
@@ -1010,12 +1055,13 @@ void Extrae_hipStreamSynchronize_Exit(void)
 		exit (-1);
 	}
 
-	Probe_HIP_StreamBarrier_Exit ();
-
-	Backend_Leave_Instrumentation ();
+	Probe_Hip_StreamBarrier_Exit ();
 }
 
-void Extrae_hipMemcpy_Enter(void* p1, const void* p2, size_t p3, enum hipMemcpyKind p4)
+void Extrae_hipMemcpy_Enter(void* p1, 
+	const void* p2, 
+	size_t p3, 
+	enum hipMemcpyKind p4)
 {
 	int devid;
 	unsigned tag;
@@ -1031,8 +1077,7 @@ void Extrae_hipMemcpy_Enter(void* p1, const void* p2, size_t p3, enum hipMemcpyK
 	hipGetDevice (&devid);
 	Extrae_HIP_Initialize (devid);
 
-	Backend_Enter_Instrumentation ();
-	Probe_HIP_Memcpy_Entry (p3);
+	Probe_Hip_Memcpy_Entry (p3);
 
 	tag = Extrae_HIP_tag_generator();
 
@@ -1051,13 +1096,27 @@ void Extrae_hipMemcpy_Enter(void* p1, const void* p2, size_t p3, enum hipMemcpyK
 	 */
 	if (p4 == hipMemcpyHostToDevice)
 	{
-		Extrae_HIP_AddEventToStream(EXTRAE_HIP_NEW_TIME, devid, 0,
-		  HIPMEMCPY_GPU_VAL, EVT_BEGIN, 0, p3);
+		Extrae_HIP_AddEventToStream(EXTRAE_HIP_NEW_TIME, 
+			devid, 
+			0,
+			HIPMEMCPY_GPU_VAL, 
+			EVT_BEGIN, 
+			0, 
+			p3, 
+			0, 0, 0, 
+			0, 0, 0);
 	}
 	else if (p4 == hipMemcpyDeviceToHost)
 	{
-		Extrae_HIP_AddEventToStream(EXTRAE_HIP_NEW_TIME, devid, 0,
-		  HIPMEMCPY_GPU_VAL, EVT_BEGIN, tag, p3);
+		Extrae_HIP_AddEventToStream(EXTRAE_HIP_NEW_TIME, 
+			devid, 
+			0, 
+			HIPMEMCPY_GPU_VAL, 
+			EVT_BEGIN, 
+			tag, 
+			p3, 
+			0, 0, 0,
+			0, 0, 0);
 	}
 }
 
@@ -1068,8 +1127,8 @@ void Extrae_hipMemcpy_Exit(void)
 
 	ASSERT(Extrae_HIP_saved_params!=NULL, "Unallocated Extrae_HIP_saved_params");
 
-	hipGetDevice (&devid);
-	Extrae_HIP_Initialize (devid);
+	hipGetDevice(&devid);
+	Extrae_HIP_Initialize(devid);
 
 	tag = Extrae_HIP_tag_get();
 
@@ -1083,16 +1142,30 @@ void Extrae_hipMemcpy_Exit(void)
 	if (Extrae_HIP_saved_params[THREADID].punion.cm.kind == hipMemcpyHostToDevice ||
 	  Extrae_HIP_saved_params[THREADID].punion.cm.kind == hipMemcpyDeviceToDevice)
 	{
-		Extrae_HIP_AddEventToStream(EXTRAE_HIP_NEW_TIME, devid, 0,
-		  HIPMEMCPY_GPU_VAL, EVT_END, tag, Extrae_HIP_saved_params[THREADID].punion.cm.size);
+		Extrae_HIP_AddEventToStream(EXTRAE_HIP_NEW_TIME, 
+			devid, 
+			0,
+			HIPMEMCPY_GPU_VAL, 
+			EVT_END, 
+			tag, 
+			Extrae_HIP_saved_params[THREADID].punion.cm.size, 
+			0, 0, 0, 
+			0, 0, 0);
 	}
 	else if (Extrae_HIP_saved_params[THREADID].punion.cm.kind == hipMemcpyDeviceToHost)
 	{
-		Extrae_HIP_AddEventToStream(EXTRAE_HIP_NEW_TIME, devid, 0,
-		  HIPMEMCPY_GPU_VAL, EVT_END, 0, Extrae_HIP_saved_params[THREADID].punion.cm.size);
+		Extrae_HIP_AddEventToStream(EXTRAE_HIP_NEW_TIME, 
+			devid, 
+			0,
+			HIPMEMCPY_GPU_VAL, 
+			EVT_END, 
+			0, 
+			Extrae_HIP_saved_params[THREADID].punion.cm.size, 
+			0, 0, 0, 
+			0, 0, 0);
 	}
 
-	Probe_HIP_Memcpy_Exit ();
+	Probe_Hip_Memcpy_Exit ();
 
 	/* Emit communication to the host side if memcpykind refers to {host,device} to host */
 	if (Extrae_HIP_saved_params[THREADID].punion.cm.kind == hipMemcpyDeviceToHost)
@@ -1100,11 +1173,13 @@ void Extrae_hipMemcpy_Exit(void)
 		TRACE_USER_COMMUNICATION_EVENT(LAST_READ_TIME, USER_RECV_EV,
 		  TASKID, Extrae_HIP_saved_params[THREADID].punion.cm.size, tag, tag);
 	}
-
-	Backend_Leave_Instrumentation ();
 }
 
-void Extrae_hipMemcpyAsync_Enter (void* p1, const void* p2, size_t p3, enum hipMemcpyKind p4, hipStream_t p5)
+void Extrae_hipMemcpyAsync_Enter(void* p1, 
+	const void* p2, 
+	size_t p3, 
+	enum hipMemcpyKind p4, 
+	hipStream_t p5)
 {
 	int devid;
 	int strid;
@@ -1122,8 +1197,7 @@ void Extrae_hipMemcpyAsync_Enter (void* p1, const void* p2, size_t p3, enum hipM
 	hipGetDevice (&devid);
 	Extrae_HIP_Initialize (devid);
 
-	Backend_Enter_Instrumentation ();
-	Probe_HIP_MemcpyAsync_Entry (p3);
+	Probe_Hip_MemcpyAsync_Entry (p3);
 
 	tag = Extrae_HIP_tag_generator();
 
@@ -1149,17 +1223,31 @@ void Extrae_hipMemcpyAsync_Enter (void* p1, const void* p2, size_t p3, enum hipM
 	 */
 	if (p4 == hipMemcpyHostToDevice)
 	{
-		Extrae_HIP_AddEventToStream(EXTRAE_HIP_NEW_TIME, devid, strid,
-		  HIPMEMCPYASYNC_GPU_VAL, EVT_BEGIN, 0, p3);
+		Extrae_HIP_AddEventToStream(EXTRAE_HIP_NEW_TIME, 
+			devid, 
+			strid,
+			HIPMEMCPYASYNC_GPU_VAL, 
+			EVT_BEGIN, 
+			0, 
+			p3, 
+			0, 0, 0, 
+			0, 0, 0);
 	}
 	else if (p4 == hipMemcpyDeviceToHost)
 	{
-		Extrae_HIP_AddEventToStream(EXTRAE_HIP_NEW_TIME, devid, strid,
-		  HIPMEMCPYASYNC_GPU_VAL, EVT_BEGIN, tag, p3);
+		Extrae_HIP_AddEventToStream(EXTRAE_HIP_NEW_TIME, 
+			devid, 
+			strid,
+			HIPMEMCPYASYNC_GPU_VAL, 
+			EVT_BEGIN, 
+			tag, 
+			p3, 
+			0, 0, 0, 
+			0, 0, 0);
 	}
 }
 
-void Extrae_hipMemcpyAsync_Exit (void)
+void Extrae_hipMemcpyAsync_Exit(void)
 {
 	int devid;
 	int strid;
@@ -1190,16 +1278,30 @@ void Extrae_hipMemcpyAsync_Exit (void)
 	if (Extrae_HIP_saved_params[THREADID].punion.cma.kind == hipMemcpyHostToDevice ||
 	   Extrae_HIP_saved_params[THREADID].punion.cma.kind == hipMemcpyDeviceToDevice)
 	{
-		Extrae_HIP_AddEventToStream(EXTRAE_HIP_NEW_TIME, devid, strid,
-		  HIPMEMCPYASYNC_GPU_VAL, EVT_END, tag, Extrae_HIP_saved_params[THREADID].punion.cma.size);
+		Extrae_HIP_AddEventToStream(EXTRAE_HIP_NEW_TIME, 
+			devid, 
+			strid,
+			HIPMEMCPYASYNC_GPU_VAL, 
+			EVT_END, 
+			tag, 
+			Extrae_HIP_saved_params[THREADID].punion.cma.size, 
+			0, 0, 0, 
+			0, 0, 0);
 	}
 	else
 	{
-		Extrae_HIP_AddEventToStream(EXTRAE_HIP_NEW_TIME, devid, strid,
-		  HIPMEMCPYASYNC_GPU_VAL, EVT_END, 0, Extrae_HIP_saved_params[THREADID].punion.cma.size);
+		Extrae_HIP_AddEventToStream(EXTRAE_HIP_NEW_TIME, 
+			devid, 
+			strid,
+			HIPMEMCPYASYNC_GPU_VAL, 
+			EVT_END, 
+			0, 
+			Extrae_HIP_saved_params[THREADID].punion.cma.size, 
+			0,0,0, 
+			0,0,0);
 	}
 
-	Probe_HIP_MemcpyAsync_Exit ();
+	Probe_Hip_MemcpyAsync_Exit ();
 
 	/* Emit communication to the host side if memcpykind refers to {host,device} to host */
 	if (Extrae_HIP_saved_params[THREADID].punion.cma.kind == hipMemcpyDeviceToHost)
@@ -1207,19 +1309,16 @@ void Extrae_hipMemcpyAsync_Exit (void)
 		TRACE_USER_COMMUNICATION_EVENT(LAST_READ_TIME, USER_RECV_EV,
 		  TASKID, Extrae_HIP_saved_params[THREADID].punion.cma.size, tag, tag);
 	}
-
-	Backend_Leave_Instrumentation ();
 }
 
-void Extrae_hipMemset_Enter(void* devPtr, size_t count)
+void Extrae_hipMemset_Enter(void *devPtr, size_t count)
 {
 	int devid;
 
 	hipGetDevice(&devid);
 	Extrae_HIP_Initialize(devid);
 
-	Backend_Enter_Instrumentation();
-	Probe_HIP_Memset_Entry((UINT64)devPtr, count);
+	Probe_Hip_Memset_Entry((UINT64)devPtr, count);
 }
 
 void Extrae_hipMemset_Exit(void)
@@ -1229,16 +1328,15 @@ void Extrae_hipMemset_Exit(void)
 	hipGetDevice(&devid);
 	Extrae_HIP_Initialize(devid);
 
-	Probe_HIP_Memset_Exit();
-	Backend_Leave_Instrumentation();
+	Probe_Hip_Memset_Exit();
 }
 
 void Extrae_reallocate_HIP_info(unsigned old_threads, unsigned nthreads)
 {
-	Extrae_HIP_saved_params = (Extrae_HIP_saved_params_t*) xrealloc (
-		Extrae_HIP_saved_params, sizeof(Extrae_HIP_saved_params_t)*nthreads);
+	Extrae_HIP_saved_params = (Extrae_hip_saved_params_t*) xrealloc (
+		Extrae_HIP_saved_params, sizeof(Extrae_hip_saved_params_t)*nthreads);
 
-	memset(&Extrae_HIP_saved_params[old_threads], 0, sizeof(Extrae_HIP_saved_params_t)*(nthreads-old_threads));
+	memset(&Extrae_HIP_saved_params[old_threads], 0, sizeof(Extrae_hip_saved_params_t)*(nthreads-old_threads));
 
 	if (Extrae_HIP_saved_params == NULL)
 	{
@@ -1249,9 +1347,8 @@ void Extrae_reallocate_HIP_info(unsigned old_threads, unsigned nthreads)
 
 void Extrae_hipDeviceReset_Enter(void)
 {
-	Backend_Enter_Instrumentation();
-    Probe_HIP_DeviceReset_Enter();
-    Extrae_HIP_flush_streams(XTR_FLUSH_ALL_DEVICES, XTR_FLUSH_ALL_STREAMS);
+	Probe_Hip_DeviceReset_Enter();
+	Extrae_HIP_flush_streams(XTR_FLUSH_ALL_DEVICES, XTR_FLUSH_ALL_STREAMS);
 }
 
 void Extrae_hipDeviceReset_Exit(void)
@@ -1260,52 +1357,52 @@ void Extrae_hipDeviceReset_Exit(void)
 	hipGetDevice(&devid);
 
 	Extrae_HIP_deInitialize(devid);
-	Probe_HIP_DeviceReset_Exit();
-	Backend_Leave_Instrumentation ();
+	Probe_Hip_DeviceReset_Exit();
 }
 
-void Extrae_hipThreadExit_Enter(void)
+void Extrae_hipThreadExit_Enter (void)
 {
-	Backend_Enter_Instrumentation();
-	Probe_HIP_ThreadExit_Enter();
+	Probe_Hip_ThreadExit_Enter();
 }
 
-void Extrae_hipThreadExit_Exit(void)
+void Extrae_hipThreadExit_Exit (void)
 {
 	int devid;
 	hipGetDevice (&devid);
 
 	Extrae_HIP_deInitialize (devid);
-	Probe_HIP_ThreadExit_Exit();
-	Backend_Leave_Instrumentation();
+	Probe_Hip_ThreadExit_Exit();
 }
 
 void Extrae_hipEventRecord_Enter(void)
 {
-	Backend_Enter_Instrumentation();
-	Probe_HIP_EventRecord_Entry();
+	Probe_Hip_EventRecord_Entry();
 }
 
 void Extrae_hipEventRecord_Exit(void)
 {
-	Probe_HIP_EventRecord_Exit();
-	Backend_Leave_Instrumentation();
+	Probe_Hip_EventRecord_Exit();
 }
 
 void Extrae_hipEventSynchronize_Enter(void)
 {
-	Backend_Enter_Instrumentation();
-	Probe_HIP_EventSynchronize_Entry();
+	Probe_Hip_EventSynchronize_Entry();
 }
 
 void Extrae_hipEventSynchronize_Exit(void)
 {
-	Probe_HIP_EventSynchronize_Exit();
-	Backend_Leave_Instrumentation();
+	Probe_Hip_EventSynchronize_Exit();
 }
 
+/**
+ * Performing the flush of streams events during the finalization of Extrae in the destructor call
+ * triggers HIP calls after the HIP library has been unloaded resulting in crashes.
+ * To solve that case we always call Extrae_HIP_finalize with atexit and mark hipInitialized to 0
+ * to prevent entering this routine after the return point of the main.
+ */
 void Extrae_HIP_finalize(void)
 {
+	Backend_Enter_Instrumentation();
 	if (EXTRAE_INITIALIZED() && hipInitialized == 1)
 	{
 		for (int i = 0; i < HIP_MAP_NUM_BUCKETS; ++i)
@@ -1324,7 +1421,7 @@ void Extrae_HIP_finalize(void)
 						if (start) 
 							sscanf(start + 5, "%d", &lineno);
 					}
-					Extrae_AddFunctionDefinitionEntryToLocalSYM('K', 
+					Extrae_AddFunctionDefinitionEntryToLocalSYM('Y', 
 						(UINT64)n->key, 
 						n->val.name, 
 						"??", 
@@ -1334,19 +1431,19 @@ void Extrae_HIP_finalize(void)
 			}
 		}
 		
-		Extrae_HIP_flush_streams (XTR_FLUSH_ALL_DEVICES, XTR_FLUSH_ALL_STREAMS);
+		Extrae_HIP_flush_streams(XTR_FLUSH_ALL_DEVICES, XTR_FLUSH_ALL_STREAMS);
 		
 		for(int i = 0; i <HIPdevices ; ++i)
 		{
-			Extrae_HIP_deInitialize (i);
+			gpuEventList_free(&devices[i].availableEvents);
+			Extrae_HIP_deInitialize(i);
 		}
-		
 		xfree(devices);
+
 		devices = NULL;
 		HIPdevices = 0;
 
-		gpuEventList_free(&availableEvents);
-        hipmap_clear_all_safe();
 		hipInitialized = 0;
 	}
-}
+	Backend_Leave_Instrumentation();
+}
\ No newline at end of file
